@@ -1,4 +1,4 @@
-/* $Id: config.c,v 1.113.2.1 2004/04/20 05:05:27 alan Exp $ */
+/* $Id: config.c,v 1.113.2.7 2004/09/14 22:34:12 alan Exp $ */
 /*
  * Parse various heartbeat configuration files...
  *
@@ -53,6 +53,7 @@
 #include <ha_msg.h>
 #include <pils/plugin.h>
 #include <clplumbing/realtime.h>
+#include <clplumbing/netstring.h>
 #include <HBcomm.h>
 #include <hb_module.h>
 #include <hb_api.h>
@@ -87,6 +88,8 @@ static int set_debuglevel(const char *);
 static int set_normalpoll(const char *);
 static int set_api_authorization(const char *);
 static int set_msgfmt(const char*);
+static int set_register_to_apphbd(const char *);
+static int set_badpack_warn(const char*);
 
 /*
  * Each of these parameters is is automatically recorded by
@@ -122,6 +125,8 @@ struct directive {
 , {KEY_DEBUGLEVEL,set_debuglevel, TRUE, NULL, "debug level"}
 , {KEY_NORMALPOLL,set_normalpoll, TRUE, "true", "Use system poll(2) function?"}
 , {KEY_MSGFMT,    set_msgfmt, TRUE, "classic", "message format in the wire"}
+, {KEY_REGAPPHBD, set_register_to_apphbd, FALSE, NULL, "register with apphbd"}
+, {KEY_BADPACK,   set_badpack_warn, TRUE, "true", "warn about bad packets"}
 };
 
 static const struct WholeLineDirective {
@@ -157,6 +162,7 @@ extern int    				timebasedgenno;
 int    					enable_realtime = TRUE;
 extern int    				debug;
 int					netstring_format = FALSE;
+extern int				UseApphbd;
 
 static int	islegaldirective(const char *directive);
 static int	parse_config(const char * cfgfile, char *nodename);
@@ -455,6 +461,7 @@ parse_config(const char * cfgfile, char *nodename)
 	{	{"ipfail",	"uid=" HA_CCMUSER}
 	,	{"ccm",		"uid=" HA_CCMUSER}
 	,	{"ping",	"gid=" HA_APIGROUP}
+	,	{"cl_status",	"gid=" HA_APIGROUP}
 	};
 
 	if ((f = fopen(cfgfile, "r")) == NULL) {
@@ -870,6 +877,14 @@ add_option(const char *	option, const char * value)
 		funs->descr(&descr);
 		funs->mtype(&type);
 
+		if (nummedia >= MAXMEDIA) {
+			cl_log(LOG_ERR, "Too many media specified (> %d)"
+			,	MAXMEDIA);
+			cl_log(LOG_INFO, "Offending command: %s %s"
+			,	option, value);
+			return HA_FAIL;
+		}
+
 		sysmedia[nummedia] = mp;
 		if (mp == NULL) {
 			ha_log(LOG_ERR, "Illegal %s [%s] in config file [%s]"
@@ -893,10 +908,10 @@ add_option(const char *	option, const char * value)
 			++nummedia;
 			PILIncrIFRefCount(PluginLoadingSystem
 			,	HB_COMM_TYPE_S, option, +1);
-			return(HA_OK);
 		}
 		g_assert(sysmedia[nummedia-1]->type);
 		g_assert(sysmedia[nummedia-1]->description);
+		return(HA_OK);
 	}
 	ha_log(LOG_ERR, "Illegal configuration directive [%s]", option);
 	return(HA_FAIL);
@@ -1271,6 +1286,12 @@ set_auto_failback(const char * value)
 	return rc;
 }
 
+static int 
+set_register_to_apphbd(const char * value)
+{
+	return str_to_boolean(value, &UseApphbd);
+}
+
 /*
  *	Convert a string into a positive, rounded number of milliseconds.
  *
@@ -1607,6 +1628,18 @@ set_msgfmt(const char* value)
 	
 	return HA_FAIL;
 }
+static int
+set_badpack_warn(const char* value)
+{
+	int	warnme = TRUE;
+	int	rc;
+	rc = str_to_boolean(value, &warnme);
+
+	if (HA_OK == rc) {
+		cl_msg_quiet_fmterr = !warnme;
+	}
+	return rc;
+}
 
 static int
 add_client_child(const char * directive)
@@ -1669,7 +1702,7 @@ add_client_child(const char * directive)
 	command[cmdlen] = EOS;
 
 	path = ha_malloc(pathlen+1);
-	if (command == NULL) {
+	if (path == NULL) {
 		ha_log(LOG_ERR, "Out of memory in add_client_child "
 				"(path)");
 		ha_free(command); command=NULL;
@@ -1733,14 +1766,17 @@ unametonum(const char * lname, int llen)
 	char	loginname[64];
 	struct passwd*	pwd;
 
-	if (isdigit(loginname[0])) {
-		return atoi(loginname);
-	}
 	if (llen >= (int)sizeof(loginname)) {
+		cl_log(LOG_ERR 
+		,	"user id name [%s] is too long", loginname);
 		return -1;
 	}
 	strncpy(loginname, lname, llen);
 	loginname[llen] = EOS;
+
+	if (isdigit(loginname[0])) {
+		return atoi(loginname);
+	}
 	if ((pwd = getpwnam(loginname)) == NULL) {
 		cl_log(LOG_ERR 
 		,	"Invalid user id name [%s]", loginname);
@@ -1802,6 +1838,7 @@ make_id_table(const char * list, int listlen, int (*map)(const char *, int))
 		}
 		idval = map(id, idlen);
 		if (idval < 0) {
+			g_hash_table_destroy(ret);
 			return NULL;
 		}
 		if (ANYDEBUG) {
@@ -1911,7 +1948,7 @@ set_api_authorization(const char * directive)
 		goto baddirective;
 	}
 
-	memset(auth, 0, sizeof(auth));
+	memset(auth, 0, sizeof(*auth));
 
 	if (uidlist) {
 		auth->uid = make_id_table(uidlist, uidlen, unametonum);
@@ -1965,7 +2002,7 @@ baddirective:
 			g_hash_table_destroy(auth->gid);
 			auth->gid = NULL;
 		}
-		memset(auth, 0, sizeof(auth));
+		memset(auth, 0, sizeof(*auth));
 		ha_free(auth);
 		auth = NULL;
 	}
@@ -1982,6 +2019,50 @@ baddirective:
 
 /*
  * $Log: config.c,v $
+ * Revision 1.113.2.7  2004/09/14 22:34:12  alan
+ * Changed cl_status from setuid to setgid
+ *
+ * Revision 1.113.2.6  2004/09/11 20:52:32  alan
+ * Brought back changes from HEAD
+ * 1) Added apphbd option
+ * 2) Restructuring of packet handling
+ * 3) Various BEAM fixes
+ *
+ * Revision 1.128  2004/09/10 22:47:40  alan
+ * BEAM FIXES:  various minor fixes related to running out of memory.
+ *
+ * Revision 1.127  2004/09/10 07:23:50  alan
+ * BEAM fixes:  Fixed a couple of resource leaks, and a couple of use-after-free errors.
+ *
+ * Revision 1.126  2004/09/10 01:12:23  alan
+ * BEAM CHANGES: Fixed a couple of very minor bugs, and cleaned up some BEAM warnings.
+ *
+ * Revision 1.125  2004/08/31 20:58:21  alan
+ * Fixed the size of a memset call...
+ *
+ * Revision 1.124  2004/08/31 20:56:11  alan
+ * Fixed the new bug that kept heartbeat from passing BasicSanityCheck
+ *
+ * Revision 1.123  2004/08/31 18:29:15  alan
+ * added the code to config.c to suppress warnings about bad packets
+ *
+ * Revision 1.122  2004/08/31 13:47:31  alan
+ * Put in a bug fix to check for MAXMEDIA in the configuration file.
+ *
+ * Revision 1.120  2004/05/24 09:20:08  sunjd
+ * make heartbeat an apphbd client
+ *
+ * Revision 1.113.2.4  2004/08/28 04:51:33  alan
+ * Put in changes which (hopefully) will make cl_status work without any
+ * special local configuration.
+ *
+ * Revision 1.113.2.3  2004/08/28 00:29:35  alan
+ * Fixed a bug which Lars Ellenberg pointd out...
+ *
+ * Revision 1.113.2.2  2004/08/05 18:30:46  alan
+ * Put in changes to support a new option which allows one to suppress
+ * notification of packet corruption.
+ *
  * Revision 1.113.2.1  2004/04/20 05:05:27  alan
  * Backported everything from 1.3.0 to 1.2.1
  *
