@@ -1,4 +1,4 @@
-/* $Id: findif.c,v 1.32.2.2 2004/04/29 01:28:07 alan Exp $ */
+/* $Id: findif.c,v 1.32.2.4 2004/09/11 20:52:32 alan Exp $ */
 /*
  * findif.c:	Finds an interface which can route a given address
  *
@@ -126,6 +126,8 @@ void GetAddress (char *inputaddress, char **address, char **netmaskbits
 
 void ValidateNetmaskBits (char *netmaskbits, unsigned long *netmask);
 
+int netmask_bits (unsigned long netmask);
+
 const char *	cmdname = "findif";
 void usage(void);
 
@@ -428,6 +430,8 @@ ValidateNetmaskBits (char *netmaskbits, unsigned long *netmask)
 				,	"Invalid netmask specification [%s]"
 				,	netmaskbits);
 				usage();
+				/*not reached */
+				exit(1);
 			}
 
 			bits = 32 - bits;
@@ -438,6 +442,21 @@ ValidateNetmaskBits (char *netmaskbits, unsigned long *netmask)
 	}
 }
 
+int
+netmask_bits(unsigned long netmask) {
+	int	j;
+
+	netmask = netmask & 0xFFFFFFFFUL;
+
+	for (j=0; j <= 32; ++j) {
+		if ((netmask >> j)&0x1) {
+			break;
+		}
+	}
+
+	return 32 - j;
+}
+
 int
 main(int argc, char ** argv) {
 
@@ -504,6 +523,8 @@ main(int argc, char ** argv) {
 		char errmsg[MAXSTR] = "No valid mecahnisms";
 		int rc = -1;
 
+		strcpy(best_if, "UNKNOWN"); /* just in case */
+
 		while (*sr) {
 			errmsg[0] = '\0';
 			rc = (*sr) (address, &in, &addr_out, best_if
@@ -538,13 +559,20 @@ main(int argc, char ** argv) {
 	if (bcast_arg) {
 		best_netmask = htonl(best_netmask);
 		/* Yes, they gave us a broadcast address */
-		printf("%s\t netmask %d.%d.%d.%d\tbroadcast %s\n"
-		,	best_if
-                ,       (int)((best_netmask>>24) & 0xff)
-                ,       (int)((best_netmask>>16) & 0xff)
-                ,       (int)((best_netmask>>8) & 0xff)
-                ,       (int)(best_netmask & 0xff)
-		,	bcast_arg);
+		if (!OutputInCIDR) {
+			printf("%s\t netmask %d.%d.%d.%d\tbroadcast %s\n"
+			,	best_if
+                	,       (int)((best_netmask>>24) & 0xff)
+                	,       (int)((best_netmask>>16) & 0xff)
+                	,       (int)((best_netmask>>8) & 0xff)
+                	,       (int)(best_netmask & 0xff)
+			,	bcast_arg);
+		}else{
+			printf("%s\t netmask %d\tbroadcast %s\n"
+			,	best_if
+			,	netmask_bits(best_netmask)
+			,	bcast_arg);
+		}
 	}else{
 		/* No, we use a common broadcast address convention */
 		unsigned long	def_bcast;
@@ -571,18 +599,9 @@ main(int argc, char ** argv) {
 			,       (int)((def_bcast>>8) & 0xff)
 			,       (int)(def_bcast & 0xff));
 		}else{
-			int	j;
-
-			best_netmask = best_netmask & 0xFFFFFFFFUL;
-
-			for (j=0; j <= 32; ++j) {
-				if ((best_netmask >> j)&0x1) {
-					break;
-				}
-			}
 			printf("%s\tnetmask %d\tbroadcast %d.%d.%d.%d\n"
 			,       best_if
-			,	32-j
+			,	netmask_bits(best_netmask)
 			,       (int)((def_bcast>>24) & 0xff)
 			,       (int)((def_bcast>>16) & 0xff)
 			,       (int)((def_bcast>>8) & 0xff)
@@ -607,6 +626,10 @@ usage()
 		"    interface: interface to add the virtual interface to\n"
 		"    broadcast: broadcast address of the network that "
 			"address belongs to\n"
+		"\n"
+		"Options:\n"
+		"    -C: Output netmask as the number of bits rather "
+			"than as 4 octets.\n"
 	,	cmdname, cmdname);
 	exit(1);
 }
@@ -679,11 +702,26 @@ ff02::%lo0/32                     fe80::1%lo0                   UC          lo0
 
 /* 
  * $Log: findif.c,v $
- * Revision 1.32.2.2  2004/04/29 01:28:07  alan
- * Put in a small fix to findif provided by Lars Ellenberg.
+ * Revision 1.32.2.4  2004/09/11 20:52:32  alan
+ * Brought back changes from HEAD
+ * 1) Added apphbd option
+ * 2) Restructuring of packet handling
+ * 3) Various BEAM fixes
  *
- * Revision 1.32.2.1  2004/04/20 05:05:27  alan
- * Backported everything from 1.3.0 to 1.2.1
+ * Revision 1.32.2.3  2004/08/29 08:14:23  horms
+ * Backport sendarp updates to 1.2
+ *
+ * Revision 1.38  2004/04/29 14:18:20  kevin
+ * Be nice to the non C99 compliant comilers. (Declarations come first)
+ *
+ * Revision 1.37  2004/04/29 07:12:19  lars
+ * add missing return
+ *
+ * Revision 1.36  2004/04/27 11:33:19  horms
+ * Honour -C flag when broadcast is supplied
+ *
+ * Revision 1.35  2004/04/27 11:25:59  horms
+ * Document -C option to findif
  *
  * Revision 1.34  2004/03/25 07:55:39  alan
  * Moved heartbeat libraries to the lib directory.
