@@ -1,4 +1,4 @@
-/* $Id: ha_msg_internal.c,v 1.39.2.1 2004/04/20 05:05:27 alan Exp $ */
+/* $Id: ha_msg_internal.c,v 1.39.2.2 2004/08/05 18:30:46 alan Exp $ */
 /*
  * ha_msg_internal: heartbeat internal messaging functions
  *
@@ -32,6 +32,7 @@
 #include <heartbeat.h>
 #include <ha_msg.h>
 #include <heartbeat_private.h>
+#include <clplumbing/netstring.h>
 
 #define		MINFIELDS	30
 #define		CRNL		"\r\n"
@@ -219,7 +220,11 @@ isauthentic(const struct ha_msg * m)
 	
 	if (authtoken == NULL
 	||	sscanf(authtoken, "%d %s", &authwhich, authstring) != 2) {
-		ha_log(LOG_WARNING, "Bad/invalid auth token, authtoken=%p", authtoken);
+		if (!cl_msg_quiet_fmterr) {
+			ha_log(LOG_WARNING
+			,	"Bad/invalid auth token, authtoken=%p"
+			,	authtoken);
+		}
 		return(0);
 	}
 	which = config->auth_config + authwhich;
@@ -352,6 +357,10 @@ main(int argc, char ** argv)
 #endif
 /*
  * $Log: ha_msg_internal.c,v $
+ * Revision 1.39.2.2  2004/08/05 18:30:46  alan
+ * Put in changes to support a new option which allows one to suppress
+ * notification of packet corruption.
+ *
  * Revision 1.39.2.1  2004/04/20 05:05:27  alan
  * Backported everything from 1.3.0 to 1.2.1
  *
