@@ -1,4 +1,4 @@
-/* $Id: hb_api.c,v 1.94.2.1 2004/04/20 05:05:27 alan Exp $ */
+/* $Id: hb_api.c,v 1.94.2.3 2004/09/11 20:52:32 alan Exp $ */
 /*
  * hb_api: Server-side heartbeat API code
  *
@@ -459,7 +459,7 @@ api_iflist(const struct ha_msg* msg, struct ha_msg* resp
 		}
 
 		/* Find last link... */
- 		for(j=0; (lnk = &node->links[j]) && lnk->name; ++j) {
+ 		for(j=0; (lnk = &node->links[j], lnk->name); ++j) {
 			last = j;
                 }
 		/* Don't report on ping links */
@@ -499,7 +499,7 @@ api_ping_iflist(const struct ha_msg* msg, struct node_info * node
 	int	j;
 	struct link * lnk;
 
- 	for(j=0; (lnk = &node->links[j]) && lnk->name; ++j) {
+ 	for(j=0; (lnk = &node->links[j], lnk->name); ++j) {
 		if (strcmp(lnk->name, node->nodename) == 0) {
 			if (ha_msg_mod(resp, F_IFNAME
 			,	lnk->name) != HA_OK) {
@@ -843,7 +843,9 @@ freeandexitresp:
 	ha_msg_del(resp);
 	resp=NULL;
 freeandexit:
-	ha_msg_del(msg); msg=NULL;
+	if (msg != NULL) {
+		ha_msg_del(msg); msg=NULL;
+	}
 }
 
 /* Process a registration request from a potential client */
@@ -897,7 +899,7 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	const char *	reqtype;
 	const char *	fromid;
 	const char *	pid;
-	struct ha_msg *	resp;
+	struct ha_msg *	resp = NULL;
 	client_proc_t*	fcli;
 	
 	char		deadtime[64];
@@ -911,7 +913,7 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	||	strcmp(reqtype, API_SIGNON) != 0)  {
 		cl_log(LOG_ERR, "api_process_registration_msg: bad message");
 		cl_log_message(msg);
-		return;
+		goto del_msg;
 	}
 	fromid = ha_msg_value(msg, F_FROMID);
 	pid = ha_msg_value(msg, F_PID);
@@ -919,7 +921,7 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	if (fromid == NULL && pid == NULL) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: no fromid in msg");
-		return;
+		goto del_msg;
 	}
 	if (DEBUGDETAILS) {
 		cl_log(LOG_DEBUG
@@ -933,19 +935,17 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	if ((resp = ha_msg_new(4)) == NULL) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: out of memory/1");
-		return;
+		goto del_msg;
 	}
 	if (ha_msg_add(resp, F_TYPE, T_APIRESP) != HA_OK) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: cannot add field/2");
-		ha_msg_del(resp); resp=NULL;
-		return;
+		goto del_rsp_and_msg;
 	}
 	if (ha_msg_add(resp, F_APIREQ, reqtype) != HA_OK) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: cannot add field/3");
-		ha_msg_del(resp); resp=NULL;
-		return;
+		goto del_rsp_and_msg;
 	}
 
 	client->pid = atoi(pid);
@@ -961,20 +961,19 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	if ((fcli = find_client(fromid, pid)) == NULL) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: cannot find client");
-		ha_msg_del(resp); resp=NULL;
 		/* We can't properly reply to them. They'll hang. Sorry... */
-		return;
+		goto del_rsp_and_msg;
 	}
 	if (fcli != client) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: found wrong client");
-		ha_msg_del(resp); resp=NULL;
+		goto del_rsp_and_msg;
 		return;
 	}
 	if (ha_msg_mod(resp, F_APIRESULT, API_OK) != HA_OK) {
 		cl_log(LOG_ERR
 		,	"api_process_registration_msg: cannot add field/4");
-		ha_msg_del(resp); resp=NULL;
+		goto del_rsp_and_msg;
 		return;
 	}
 
@@ -988,8 +987,7 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 	||	(ha_msg_mod(resp, F_NODENAME, localnodename) != HA_OK)
 	|| 	(ha_msg_add(resp, F_LOGFACILITY, logfacility) != HA_OK)) {
 		cl_log(LOG_ERR, "api_process_registration_msg: cannot add field/4");
-		ha_msg_del(resp); resp=NULL;
-		return;
+		goto del_rsp_and_msg;
 	}
 	if (ANYDEBUG) {
 		cl_log(LOG_DEBUG, "Signing on API client %ld (%s)"
@@ -997,7 +995,20 @@ api_process_registration_msg(client_proc_t* client, struct ha_msg * msg)
 		,	(client->iscasual? "'casual'" : client->client_id));
 	}
 	api_send_client_msg(client, resp);
-	ha_msg_del(resp); resp=NULL;
+	/* What to do with 'resp'.  Do we need it any more?? FIXME!! */
+	/* I think we don't need it anymore when go here, or memory leak
+	   will be very severe. You don't test it;-)	
+	goto del_msg;
+	*/
+
+del_rsp_and_msg:
+	if (resp != NULL) {
+		ha_msg_del(resp); resp=NULL;
+	}
+del_msg:
+	if (msg != NULL) {
+		ha_msg_del(msg); msg=NULL;
+	}
 }
 
 static void
@@ -1075,6 +1086,7 @@ api_remove_client_pid(pid_t c_pid, const char * reason)
 
 	client->removereason = reason;
 	G_main_del_IPC_Channel(client->gsource);
+	client->gsource = NULL;
 	return 1;
 }
 static void
@@ -1135,10 +1147,13 @@ api_remove_client_int(client_proc_t* req, const char * reason)
 				prev->next = client->next;
 			}
 
-			/* Channel is automatically destroyed 
-			 * by the G_CH* code...
-			 */
-			client->chan = NULL;
+
+#if MAKEITCRASH
+			/* Drop the source - that will destroy the 'chan' */
+			if (client->gsource) {
+				G_main_del_IPC_Channel(client->gsource);
+			}
+#endif
 
 			/* Zap! */
 			memset(client, 0, sizeof(*client));
@@ -1205,9 +1220,9 @@ api_add_client(client_proc_t* client, struct ha_msg* msg)
 	const char*	cpid;
 	const char *	fromid;
 	const char *	cgid = NULL;
-	const char *	cuid;
-	long		luid;
-	long		lgid;
+	const char *	cuid = NULL;
+	long		luid = -1;
+	long		lgid = -1;
 	uid_t		uid = (uid_t)-1;
 	gid_t		gid = (gid_t)-1;
 
@@ -1255,7 +1270,8 @@ api_add_client(client_proc_t* client, struct ha_msg* msg)
 		client->removereason = "invalid id info";
 		cl_log(LOG_ERR, "Client user/group id is incorrect"
 		" [%s] => %ld [%s] => %ld"
-		,	cuid, luid, cgid, lgid);
+		,	cuid == NULL ? "<null>" : cuid, luid
+		,	cgid == NULL ? "<null>" : cgid, lgid);
 		return FALSE;
 	}
 
@@ -1301,6 +1317,10 @@ api_check_client_authorization(client_proc_t* client)
 		
 	}
 	auth = gauth;
+	if ((long)auth->gid == (long)-1L) {
+		cl_log(LOG_DEBUG, "Darn!  -1 gid ptr in api_check_client_authorization");
+		abort();
+	}
 	if (ANYDEBUG) {
 
 		cl_log(LOG_DEBUG
