@@ -1,4 +1,4 @@
-/* $Id: hb_resource.c,v 1.48.2.2 2004/04/29 01:08:58 alan Exp $ */
+/* $Id: hb_resource.c,v 1.48.2.11 2004/09/11 20:52:33 alan Exp $ */
 /*
  * hb_resource: Linux-HA heartbeat resource management code
  *
@@ -169,6 +169,67 @@ static ProcTrack_ops StonithStatProcessTrackOps = {
 	StonithStatProcessName
 };
 
+
+void
+init_resource_module(void)
+{
+	hb_register_comm_up_callback(comm_up_resource_action);
+}
+
+static void
+HBDoMsg_T_STARTING_or_RESOURCES(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	/*
+	 * process_resources() will deal with T_STARTING
+	 * and T_RESOURCES messages appropriately.
+	 */
+	process_resources(type, msg, fromnode);
+	heartbeat_monitor(msg, KEEPIT, iface);
+}
+
+/* Someone wants to go standby!!! */
+static void
+HBDoMsg_T_ASKRESOURCES(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	heartbeat_monitor(msg, KEEPIT, iface);
+	ask_for_resources(msg);
+}
+
+static void
+HBDoMsg_T_ASKRELEASE(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	heartbeat_monitor(msg, KEEPIT, iface);
+	if (fromnode != curnode) {
+		/*
+		 * Queue for later handling...
+		 */
+		QueueRemoteRscReq(PerformQueuedNotifyWorld, msg);
+	}
+}
+
+static void
+HBDoMsg_T_ACKRELEASE(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	/* Ignore this if we're shutting down! */
+	if (shutdown_in_progress) {
+		return;
+	}
+	heartbeat_monitor(msg, KEEPIT, iface);
+	QueueRemoteRscReq(PerformQueuedNotifyWorld, msg);
+}
+/* Process a message no one recognizes */
+static void
+HBDoMsg_default(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	heartbeat_monitor(msg, KEEPIT, iface);
+	QueueRemoteRscReq(PerformQueuedNotifyWorld, msg);
+}
+
 static const char *	rsc_msg[] =	{HB_NO_RESOURCES, HB_LOCAL_RESOURCES
 ,	HB_FOREIGN_RESOURCES, HB_ALL_RESOURCES};
 
@@ -276,7 +337,10 @@ notify_world(struct ha_msg * msg, const char * ostatus)
 	char *	const	argv[MAXFIELDS+3] = {rc_arg0, command, NULL};
 	const char *	fp;
 	char *		tp;
-	int		pid, status;
+	int		pid;
+#if WAITFORCOMMANDS
+	int		status;
+#endif
 
 	if (!DoManageResources) {
 		return;
@@ -318,7 +382,7 @@ notify_world(struct ha_msg * msg, const char * ostatus)
 
 		case 0:	{	/* Child */
 				int	j;
-				cl_make_normaltime();
+				hb_setup_child();
 				set_proc_title("%s: notify_world()", cmdname);
 				setpgid(0,0);
 				CL_SIGACTION(SIGCHLD, NULL, &sa);
@@ -368,8 +432,6 @@ notify_world(struct ha_msg * msg, const char * ostatus)
 
 #if WAITFORCOMMANDS
 	waitpid(pid, &status, 0);
-#else
-	(void)status;
 #endif
 }
 
@@ -400,7 +462,7 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 
 
 		case HB_R_SHUTDOWN:
-		case HB_R_STABLE:	break;
+		case HB_R_STABLE:	return;
 					
 		default:	
 				cl_log(LOG_ERR
@@ -442,14 +504,28 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 			send_stonith_msg(hip->nodename, T_STONITH_NOTCONFGD);
 			cl_log(LOG_WARNING, "No STONITH device configured.");
 			cl_log(LOG_WARNING, "Shared disks are not protected.");
+			/* nice_failback needs us to do this anyway... */
+			takeover_from_node(hip->nodename);
 		}
 	}else{
-		cl_log(LOG_INFO, "Dead node %s held no resources."
+		cl_log(LOG_INFO, "Dead node %s gave up resources."
 		,	hip->nodename);
 		send_stonith_msg(hip->nodename, T_STONITH_UNNEEDED);
+		if (nice_failback) {
+			/* These might happen due to timing weirdnesses */
+			if (! (procinfo->i_hold_resources & HB_LOCAL_RSC)){
+				req_our_resources(TRUE);
+			}
+			if (! (procinfo->i_hold_resources & HB_FOREIGN_RSC)){
+				takeover_from_node(hip->nodename);
+			}
+		}else{
+			/* With nice_failback disabled, we always
+			 * own our own (local) resources
+			 */
+			takeover_from_node(hip->nodename);
+		}
 	}
-	/* nice_failback needs us to do this anyway... */
-	takeover_from_node(hip->nodename);
 }
 
 static gboolean
@@ -459,7 +535,8 @@ hb_rsc_isstable(void)
 	if (!nice_failback) {
 		if (ANYDEBUG) {
 			cl_log(LOG_DEBUG
-			,	"hb_rsc_isstable: ResourceMgmt_child_count: %d"
+			,	"hb_rsc_isstable"
+			": ResourceMgmt_child_count: %d"
 			,	ResourceMgmt_child_count);
 		}
 		return ResourceMgmt_child_count == 0;
@@ -572,6 +649,14 @@ comm_up_resource_action(void)
 	static int	resources_requested_yet = 0;
 	int		deadcount = countbystatus(DEADSTATUS, TRUE);
 
+
+	hb_register_msg_callback(T_STARTING,		HBDoMsg_T_STARTING_or_RESOURCES);
+	hb_register_msg_callback(T_RESOURCES,		HBDoMsg_T_STARTING_or_RESOURCES);
+	hb_register_msg_callback(T_ASKRESOURCES,	HBDoMsg_T_ASKRESOURCES);
+	hb_register_msg_callback(T_ASKRELEASE,		HBDoMsg_T_ASKRELEASE);
+	hb_register_msg_callback(T_ACKRELEASE,		HBDoMsg_T_ACKRELEASE);
+	hb_register_msg_callback("",			HBDoMsg_default);
+
 	if (deadcount == 0) {
 		/*
 		 * If all nodes are up, we won't have to acquire
@@ -626,11 +711,7 @@ process_resources(const char * type, struct ha_msg* msg
 	static int			first_time = 1;
 
 	shutdown_if_needed();
-	if (!DoManageResources) {
-		return;
-	}
-
-	if (!nice_failback) {
+	if (!DoManageResources || !nice_failback) {
 		return;
 	}
 
@@ -671,7 +752,7 @@ process_resources(const char * type, struct ha_msg* msg
 			return;
 
 		}
-		other_is_stable = 0;
+		other_is_stable = FALSE;
 		if (ANYDEBUG) {
 			cl_log(LOG_DEBUG
 			, "process_resources: other now unstable");
@@ -755,12 +836,12 @@ process_resources(const char * type, struct ha_msg* msg
 						cl_log(LOG_INFO
 						,	"remote resource"
 						" transition completed.");
-						other_is_stable = 1;
+						other_is_stable = TRUE;
 					 	hb_send_resources_held(resourcestate == HB_R_STABLE, NULL);	
 						PerformAutoFailback();
 					}
 				}else{
-					other_is_stable = 0;
+					other_is_stable = FALSE;
 					if (ANYDEBUG) {
 						cl_log(LOG_DEBUG
 						, "process_resources(2): %s"
@@ -853,13 +934,38 @@ process_resources(const char * type, struct ha_msg* msg
 	}
 	if (strcasecmp(type, T_SHUTDONE) == 0) {
 		if (thisnode != curnode) {
-			other_is_stable = 0;
+			/*
+			 * It seems other_is_stable should be set to TRUE
+			 * when we come here because the other side
+			 * declared they are shutting down and no longer
+			 * own any resources.
+			 */
+			other_is_stable = TRUE;
 			other_holds_resources = HB_NO_RSC;
 			if (ANYDEBUG) {
 				cl_log(LOG_DEBUG
 				, "process_resources(4): %s"
 				, " other now stable - T_SHUTDONE");
 			}
+			if ((procinfo->i_hold_resources != HB_ALL_RSC)
+			&&	!shutdown_in_progress) {
+				int	rtype;
+			
+				switch (procinfo->i_hold_resources) {
+					case HB_FOREIGN_RSC:
+						rtype = HB_LOCAL_RSC;	break;
+					case HB_LOCAL_RSC:
+						rtype = HB_FOREIGN_RSC;	break;
+					default:
+					case HB_NO_RSC:
+						rtype = HB_ALL_RSC;	break;
+				}
+				
+				/* Take over resources immediately */
+				going_standby = DONE;
+				go_standby(OTHER, rtype);
+			}
+
 		}else{
 			resourcestate = newrstate = HB_R_SHUTDOWN;
 			procinfo->i_hold_resources = 0;
@@ -1096,7 +1202,7 @@ takeover_from_node(const char * nodename)
 			procinfo->i_hold_resources |= HB_FOREIGN_RSC;
 
 			other_holds_resources = HB_NO_RSC;
-			other_is_stable = 1;	/* Not going anywhere */
+			other_is_stable = TRUE;	/* Not going anywhere */
 			takeover_in_progress = TRUE;
 			if (ANYDEBUG) {
 				cl_log(LOG_DEBUG
@@ -1137,6 +1243,10 @@ req_our_resources(int getthemanyway)
 	int	pid;
 	int	upcount;
 
+	if (!DoManageResources || shutdown_in_progress) {
+		return;
+	}
+
 	if (nice_failback) {
 
 		if (((other_holds_resources & HB_FOREIGN_RSC) != 0
@@ -1185,8 +1295,7 @@ req_our_resources(int getthemanyway)
 				break;
 	}
 
-	hb_close_watchdog();
-	cl_make_normaltime();
+	hb_setup_child();
 	set_proc_title("%s: req_our_resources()", cmdname);
 	setpgid(0,0);
 	CL_SIGNAL(SIGCHLD, SIG_DFL);
@@ -1203,6 +1312,10 @@ req_our_resources(int getthemanyway)
 		setenv(HANICEFAILBACK, "yes", 1);
 	}
 	sprintf(cmd, HALIB "/ResourceManager listkeys %s", curnode->nodename);
+	if (ANYDEBUG) {
+		cl_log(LOG_DEBUG, "req_our_resources(%s)"
+		,	cmd);
+	}
 
 	if ((rkeys = popen(cmd, "r")) == NULL) {
 		cl_log(LOG_ERR, "Cannot run command %s", cmd);
@@ -1211,8 +1324,15 @@ req_our_resources(int getthemanyway)
 
 
 	for (;;) {
+		if (DEBUGDETAILS) {
+			cl_log(LOG_DEBUG, "req_our_resources() before fgets()");
+		}
 		errno = 0;
 		if (fgets(buf, MAXLINE, rkeys) == NULL) {
+			if (DEBUGDETAILS) {
+				cl_log(LOG_DEBUG
+				,	"req_our_resources() fgets => NULL");
+			}
 			if (ferror(rkeys)) {
 				cl_perror("req_our_resources: fgets failure");
 			}
@@ -1223,17 +1343,18 @@ req_our_resources(int getthemanyway)
 		if (buf[strlen(buf)-1] == '\n') {
 			buf[strlen(buf)-1] = EOS;
 		}
-		if (ANYDEBUG) {
-			cl_log(LOG_INFO, "req_our_resources()"
-			": " HALIB "/req_resource %s", buf);
-		}
 		sprintf(getcmd, HALIB "/req_resource %s", buf);
+		if (ANYDEBUG) {
+			cl_log(LOG_DEBUG, "req_our_resources()"
+			": running [%s]",	getcmd);
+		}
 		if ((rc=system(getcmd)) != 0) {
 			cl_perror("%s returned %d", getcmd, rc);
 			finalrc=HA_FAIL;
 		}
 	}
 	rc=pclose(rkeys);
+	rkeys = NULL;
 	if (rc < 0 && errno != ECHILD) {
 		cl_perror("pclose(%s) returned %d", cmd, rc);
 	}else if (rc > 0) {
@@ -1310,8 +1431,10 @@ send_stonith_msg(const char *nodename, const char *result)
 		cl_log(LOG_ERR
 		,	"Cannot send reset reply message [%s] for %s", result
 		,	nodename);
-		ha_msg_del(hmsg);
-		hmsg = NULL;
+		if (hmsg != NULL) {
+			ha_msg_del(hmsg);
+			hmsg = NULL;
+		}
 	}
 	return;
 }
@@ -1341,6 +1464,13 @@ ask_for_resources(struct ha_msg *msg)
 		,	"Standby mode only implemented when nice_failback on");
 		return;
 	}
+	if (resourcestate == HB_R_SHUTDOWN) {
+		if (ANYDEBUG){
+			cl_log(LOG_DEBUG
+			,	"standby message ignored during shutdown");
+		}
+		return;
+	}
 	info = ha_msg_value(msg, F_COMMENT);
 	from = ha_msg_value(msg, F_ORIG);
 	rsctype=ha_msg_value(msg, F_RESOURCES);
@@ -1401,7 +1531,7 @@ ask_for_resources(struct ha_msg *msg)
 				cl_log(LOG_DEBUG
 				, "ask_for_resources: other now unstable");
 			}
-			other_is_stable = 0;
+			other_is_stable = FALSE;
 			cl_log(LOG_INFO, "%s wants to go standby [%s]"
 			,	from, decode_resources(rtype));
 			if (msgfromme) {
@@ -1580,7 +1710,7 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 	 * our resources.
 	 */
 	if (who == ME) {
-		other_is_stable = 0;
+		other_is_stable = FALSE;
 		if (ANYDEBUG) {
 			cl_log(LOG_DEBUG, "go_standby: other is unstable");
 		}
@@ -1606,7 +1736,7 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 				return;
 
 				/*
-				 * We cant't block here, because then we
+				 * We can't block here, because then we
 				 * aren't sending heartbeats out...
 				 */
 		default:	
@@ -1617,8 +1747,7 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 				break;
 	}
 
-	hb_close_watchdog();
-	cl_make_normaltime();
+	hb_setup_child();
 	setpgid(0,0);
 	CL_SIGNAL(SIGCHLD, SIG_DFL);
 
@@ -1786,8 +1915,7 @@ hb_giveup_resources(void)
 				break;
 	}
 
-	hb_close_watchdog();
-	cl_make_normaltime();
+	hb_setup_child();
 	setpgid(0,0);
 	set_proc_title("%s: hb_signal_giveup_resources()", cmdname);
 
@@ -1884,9 +2012,8 @@ Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset)
 				break;
 
 	}
-	hb_close_watchdog();
-	/* Guard against possibly hanging Stonith code... */
-	cl_make_normaltime();
+	/* Guard against possibly hanging Stonith code, etc... */
+	hb_setup_child();
 	setpgid(0,0);
 	set_proc_title("%s: Initiate_Reset()", cmdname);
 	CL_SIGNAL(SIGCHLD,SIG_DFL);
@@ -2045,6 +2172,9 @@ QueueRemoteRscReq(RemoteRscReqFunc func, struct ha_msg* msg)
 	GHook*		hook;
 	const char *	fp;
 
+	if (!DoManageResources) {
+		return;
+	}
 	InitRemoteRscReqQueue();
 	hook = g_hook_alloc(&RemoteRscReqQueue);
 
@@ -2063,6 +2193,7 @@ QueueRemoteRscReq(RemoteRscReqFunc func, struct ha_msg* msg)
 			,	"%s: child process unneeded.", fp);
 			cl_log_message(msg);
 		}
+		g_hook_free(&RemoteRscReqQueue, hook);
 		return;
 	}
 
@@ -2106,8 +2237,8 @@ StartNextRemoteRscReq(void)
 	}
 	/* Call the hook... */
 	func(hook);
-	g_hook_unref(&RemoteRscReqQueue, hook);
 	g_hook_destroy_link(&RemoteRscReqQueue, hook);
+	g_hook_unref(&RemoteRscReqQueue, hook);
 }
 
 
@@ -2171,7 +2302,7 @@ StonithProcessName(ProcTrack* p)
 }
 
 static gboolean
-StonithStatProc(gpointer gph)
+StonithStatProc(gpointer dummy)
 {
 	Initiate_Reset(config->stonith, "?", FALSE);
 	return FALSE;
@@ -2187,7 +2318,7 @@ StonithStatProcessDied(ProcTrack* p, int status, int signo, int exitcode, int wa
 	}
 	g_free(h->nodename);	h->nodename=NULL;
 	g_free(p->privatedata);	p->privatedata = NULL;
-	Gmain_timeout_add(3600*1000, StonithStatProc, h);
+	Gmain_timeout_add(3600*1000, StonithStatProc, NULL);
 }
 
 static const char *
@@ -2201,6 +2332,65 @@ StonithStatProcessName(ProcTrack* p)
 
 /*
  * $Log: hb_resource.c,v $
+ * Revision 1.48.2.11  2004/09/11 20:52:33  alan
+ * Brought back changes from HEAD
+ * 1) Added apphbd option
+ * 2) Restructuring of packet handling
+ * 3) Various BEAM fixes
+ *
+ * Revision 1.48.2.10  2004/09/07 16:56:25  alan
+ * Changed some of the hb_resource code to not be active when resource management
+ * is shut off.
+ *
+ * Revision 1.48.2.9  2004/09/07 15:12:29  alan
+ * Fixed some holes in making sure things stop correctly, and also
+ * that audits don't complain, and that nothing gets acquired while
+ * shutting down.  Changes are as follows:
+ *
+ * Fixed code so that it would not try and take over when the state was
+ * STABLE or SHUTDOWN when it received a node death indication.
+ *
+ * Changed code so that it won't try and take something over that we
+ * already know we own (in most cases).
+ *
+ * When we receive a T_SHUTDONE message from our peer we now take over
+ * all the resources they had *immediately* without waiting for them
+ * to be declared dead.  This was needed to make an audit error go away.
+ * Otherwise in this interval between the other node giving up all resources
+ * and it being declared dead, we noticed that some resources weren't being
+ * served, and considered this an error.  We use the go_standby() code
+ * to do this.  The messages are a bit odd, but it works nicely.
+ * Fixing the messages so it said T_SHUTDOWN request instead of go_standby()
+ * request would be nice.  This also shortens the interval during which
+ * we don't have resources running during a graceful shutdown of one
+ * of the nodes.
+ *
+ * Ignore req_our_resources() and standby messages during shutdown.
+ *
+ * Revision 1.48.2.8  2004/09/06 22:43:08  alan
+ * Changed hb_resource so it will mark the other side as stable
+ * when it says it's given up all resources.
+ *
+ * Revision 1.48.2.7  2004/09/06 17:15:27  alan
+ * Changed CPUlimit code to think in terms of 60 second minimums, and also
+ * upped CPU limit for heartbeat.
+ * Changed heartbeat to not take over resources when there aren't any
+ * to take over (hopefully)
+ *
+ * Revision 1.48.2.6  2004/09/03 01:42:34  alan
+ * Removed a slightly dangerous piece of debugging code...
+ *
+ * Revision 1.48.2.5  2004/09/03 00:56:39  alan
+ * Added some debugging code to hb_resource.c for the really weird thing
+ * I've been seeing where fgets whacks out...
+ *
+ * Revision 1.48.2.4  2004/08/27 19:51:07  alan
+ * Put in some better messages to help debug the issue with FIFO writes crapping out,
+ * and also to neaten things up for child processes.
+ *
+ * Revision 1.48.2.3  2004/07/13 07:41:16  sunjd
+ * fix simultaneous stop issue
+ *
  * Revision 1.48.2.2  2004/04/29 01:08:58  alan
  * Put in Gshi's fix for restarting:
  * send a stability message back to the sender upon receiving that kind of message.
