@@ -1,4 +1,4 @@
-/* $Id: send_arp.c,v 1.7.2.1 2004/04/20 05:05:29 alan Exp $ */
+/* $Id: send_arp.c,v 1.7.2.3 2004/09/14 15:42:32 alan Exp $ */
 /* 
  * send_arp
  * 
@@ -206,9 +206,15 @@ main(int argc, char *argv[])
 	for (j=0; j < repeatcount; ++j) {
 		c = send_arp(l, ip, device, src_mac, broadcast, 
 				netmask, ARPOP_REQUEST);
+		if (c < 0) {
+			break;
+		}
 		mssleep(msinterval / 2);
 		c = send_arp(l, ip, device, src_mac, broadcast, 
 				netmask, ARPOP_REPLY);
+		if (c < 0) {
+			break;
+		}
 		if (j != repeatcount-1) {
 			mssleep(msinterval / 2);
 		}
@@ -297,13 +303,54 @@ get_hw_addr(char *device, u_char mac[6])
 }
 #endif
 
+
+/*
+ * Notes on send_arp() behaviour. Horms, 15th June 2004
+ *
+ * 1. Target Hardware Address
+ *    (In the ARP portion of the packet)
+ *
+ *    a) ARP Reply
+ *
+ *       Set to the MAC address we want associated with the VIP,
+ *       as per RFC2002 (4.6).
+ *
+ *       Previously set to ff:ff:ff:ff:ff:ff
+ *
+ *    b) ARP Request
+ *
+ *       Set to 00:00:00:00:00:00. According to RFC2002 (4.6)
+ *       this value is not used in an ARP request, so the value should
+ *       not matter. However, I observed that typically (always?) this value
+ *       is set to 00:00:00:00:00:00. It seems harmless enough to follow
+ *       this trend.
+ *
+ *       Previously set to ff:ff:ff:ff:ff:ff
+ *
+ *  2. Source Hardware Address
+ *     (Ethernet Header, not in the ARP portion of the packet)
+ *
+ *     Set to the MAC address of the interface that the packet is being
+ *     sent to. Actually, due to the way that send_arp is called this would
+ *     usually (always?) be the case anyway. Although this value should not
+ *     really matter, it seems sensible to set the source address to where
+ *     the packet is really coming from.  The other obvious choice would be
+ *     the MAC address that is being associated for the VIP. Which was the
+ *     previous values.  Again, these are typically the same thing.
+ *
+ *     Previously set to MAC address being associated with the VIP
+ */
+
 #ifdef HAVE_LIBNET_1_0_API
 int
 send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr, u_char *broadcast, u_char *netmask, u_short arptype)
 {
 	int n;
 	u_char *buf;
-	u_char enet_dst[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
+	u_char *target_mac;
+	u_char device_mac[6];
+	u_char bcast_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
+	u_char zero_mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
 
 
 	if (libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &buf) == -1) {
@@ -314,25 +361,54 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 	/* Convert ASCII Mac Address to 6 Hex Digits. */
 
 	/* Ethernet header */
-	libnet_build_ethernet(enet_dst, macaddr, ETHERTYPE_ARP, NULL, 0, buf);
+	if (get_hw_addr(device, device_mac) < 0) {
+		syslog(LOG_ERR, "Cannot find mac address for %s",
+				device);
+		return -1;
+	}
+
+	if (libnet_build_ethernet(bcast_mac, device_mac, ETHERTYPE_ARP, NULL, 0
+	,	buf) == -1) {
+		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		libnet_destroy_packet(&buf);
+		return -1;
+	}
+
+	if (arptype == ARPOP_REQUEST) {
+		target_mac = zero_mac;
+	}
+	else if (arptype == ARPOP_REPLY) {
+		target_mac = macaddr;
+	}
+	else {
+		syslog(LOG_ERR, "unkonwn arptype:");
+		return -1;
+	}
 
 	/*
 	 *  ARP header
 	 */
-	libnet_build_arp(ARPHRD_ETHER,	/* Hardware address type */
+	if (libnet_build_arp(ARPHRD_ETHER,	/* Hardware address type */
 		ETHERTYPE_IP,			/* Protocol address type */
 		6,				/* Hardware address length */
 		4,				/* Protocol address length */
 		arptype,			/* ARP operation */
 		macaddr,			/* Source hardware addr */
 		(u_char *)&ip,			/* Target hardware addr */
-		enet_dst,			/* Destination hw addr */
+		target_mac,			/* Destination hw addr */
 		(u_char *)&ip,			/* Target protocol address */
 		NULL,				/* Payload */
 		0,				/* Payload length */
-		buf + LIBNET_ETH_H);
+		buf + LIBNET_ETH_H) == -1) {
+	        syslog(LOG_ERR, "libnet_build_arp failed:");
+		libnet_destroy_packet(&buf);
+		return -1;
+	}
 
 	n = libnet_write_link_layer(l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
+	if (n == -1) {
+		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+	}
 
 	libnet_destroy_packet(&buf);
 	return (n);
@@ -347,31 +423,60 @@ int
 send_arp(libnet_t* lntag, u_long ip, u_char *device, u_char macaddr[6], u_char *broadcast, u_char *netmask, u_short arptype)
 {
 	int n;
-	u_char enet_dst[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
+	u_char *target_mac;
+	u_char device_mac[6];
+	u_char bcast_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
+	u_char zero_mac[6] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
+
+	if (arptype == ARPOP_REQUEST) {
+		target_mac = zero_mac;
+	}
+	else if (arptype == ARPOP_REPLY) {
+		target_mac = macaddr;
+	}
+	else {
+		syslog(LOG_ERR, "unkonwn arptype:");
+		return -1;
+	}
 
 	/*
 	 *  ARP header
 	 */
-	libnet_build_arp(ARPHRD_ETHER,	/* hardware address type */
+	if (libnet_build_arp(ARPHRD_ETHER,	/* hardware address type */
 		ETHERTYPE_IP,	/* protocol address type */
 		6,		/* Hardware address length */
 		4,		/* protocol address length */
 		arptype,	/* ARP operation type */
 		macaddr,	/* sender Hardware address */
 		(u_char *)&ip,	/* sender protocol address */
-		enet_dst,	/* target hardware address */
+		target_mac,	/* target hardware address */
 		(u_char *)&ip,	/* target protocol address */
 		NULL,		/* Payload */
 		0,		/* Length of payload */
 		lntag,		/* libnet context pointer */
 		0		/* packet id */
-	);
+	) == -1 ) {
+		syslog(LOG_ERR, "libnet_build_arp failed:");
+		return -1;
+	}
 
 	/* Ethernet header */
-	libnet_build_ethernet(enet_dst, macaddr, ETHERTYPE_ARP, NULL, 0
-	,	lntag, 0);
+	if (get_hw_addr(device, device_mac) < 0) {
+		syslog(LOG_ERR, "Cannot find mac address for %s",
+				device);
+		return -1;
+	}
+
+	if (libnet_build_ethernet(bcast_mac, device_mac, ETHERTYPE_ARP, NULL, 0
+	,	lntag, 0) == -1 ) {
+		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+		return -1;
+	}
 
 	n = libnet_write(lntag);
+	if (n == -1) {
+		syslog(LOG_ERR, "libnet_build_ethernet failed:");
+	}
 	libnet_clear_packet(lntag);
 
 	return (n);
@@ -384,23 +489,24 @@ create_pid_directory(const char *pidfilename)
 {
 	int	status;
 	struct stat stat_buf;
+	char    *pidfilename_cpy;
 	char    *dir;
 
-	dir = strdup(pidfilename);
-	if (!dir) {
+	pidfilename_cpy = strdup(pidfilename);
+	if (!pidfilename_cpy) {
 		syslog(LOG_INFO, "Memory allocation failure: %s\n",
 				strerror(errno));
 		return -1;
 	}
-
-	dirname(dir);
+	
+	dir = dirname(pidfilename_cpy);
 
 	status = stat(dir, &stat_buf); 
 
 	if (status < 0 && errno != ENOENT && errno != ENOTDIR) {
 		syslog(LOG_INFO, "Could not stat pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
-		free(dir);
+		free(pidfilename_cpy);
 		return -1;
 	}
 	
@@ -410,17 +516,18 @@ create_pid_directory(const char *pidfilename)
 		}
 		syslog(LOG_INFO, "Pid-File directory exists but is "
 				"not a directory [%s]", dir);
-		free(dir);
+		free(pidfilename_cpy);
 		return -1;
         }
 
 	if (mkdir(dir, S_IRUSR|S_IWUSR|S_IXUSR | S_IRGRP|S_IXGRP) < 0) {
 		syslog(LOG_INFO, "Could not create pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
-		free(dir);
+		free(pidfilename_cpy);
 		return -1;
 	}
 
+	free(pidfilename_cpy);
 	return 0;
 }
 
@@ -432,7 +539,7 @@ write_pid_file(const char *pidfilename)
 	int     	pidfilefd;
 	char    	pidbuf[11];
 	unsigned long   pid;
-	size_t 		bytes;
+	ssize_t 	bytes;
 
 	if (*pidfilename != '/') {
 		syslog(LOG_INFO, "Invalid pid-file name, must begin with a "
@@ -545,6 +652,15 @@ write_pid_file(const char *pidfilename)
 
 /*
  * $Log: send_arp.c,v $
+ * Revision 1.7.2.3  2004/09/14 15:42:32  alan
+ * BEAM FIX: size_t should have been ssize_t for read return code.
+ *
+ * Revision 1.7.2.2  2004/06/15 01:59:06  horms
+ * Merged in changes from Head Branch
+ *   1.12: Changes to make gratuitous ARP Packets RFC2002 (4.6) compliant
+ *   1.11: Slightly better error checking
+ *   1.10:  Fix for pid code for FreeBSD (and others)
+ *
  * Revision 1.7.2.1  2004/04/20 05:05:29  alan
  * Backported everything from 1.3.0 to 1.2.1
  *
