@@ -494,14 +494,15 @@ assign_addr6(struct in6_addr* addr6, int prefix_len, char* if_name)
 	struct in6_ifreq ifr6;
 
 	/* Get socket first */
-	int fd;
+	int		fd;
+	struct ifreq	ifr;
+
 	fd = socket(AF_INET6, SOCK_DGRAM, 0);
 	if (fd < 0) {
 		return 1;
 	}
 
 	/* Query the index of the if */
-	struct ifreq ifr;
 	strcpy(ifr.ifr_name, if_name);
 	if (ioctl(fd, SIOGIFINDEX, &ifr) < 0) {
 		return -1;
@@ -551,7 +552,7 @@ int
 is_addr6_available(struct in6_addr* addr6)
 {
 	struct sockaddr_in6	addr;
-	struct icmp6hdr *	icmph;
+	struct icmp6hdr		icmph;
 	u_char			outpack[64];
 	int			icmp_sock;
 	int			ret;
@@ -561,13 +562,13 @@ is_addr6_available(struct in6_addr* addr6)
 	struct in6_addr		local;
 
 	icmp_sock = socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
-	icmph = (struct icmp6hdr *)outpack;
-	icmph->icmp6_type = ICMPV6_ECHO_REQUEST;
-	icmph->icmp6_code = 0;
-	icmph->icmp6_cksum = 0;
-	icmph->icmp6_sequence = htons(0);
-	icmph->icmp6_identifier = 0;
+	icmph.icmp6_type = ICMPV6_ECHO_REQUEST;
+	icmph.icmp6_code = 0;
+	icmph.icmp6_cksum = 0;
+	icmph.icmp6_sequence = htons(0);
+	icmph.icmp6_identifier = 0;
 
+	memcpy(&outpack, &icmph, sizeof(icmph));
 
 	memset(&addr, 0, sizeof(struct sockaddr_in6));
 	addr.sin6_family = AF_INET6;
@@ -670,7 +671,7 @@ write_pid_file(const char *pid_file)
 	int     	pidfilefd;
 	char    	pidbuf[11];
 	unsigned long   pid;
-	size_t 		bytes;
+	ssize_t 	bytes;
 
 	if (*pid_file != '/') {
 		cl_log(LOG_INFO, "Invalid pid-file name, must begin with a "
