@@ -1,3 +1,4 @@
+/* $Id: test.h,v 1.7.2.1 2004/04/20 05:05:28 alan Exp $ */
 /*
  * test.h: header for special test code inside of heartbeat
  *
@@ -31,12 +32,16 @@ struct TestParms {
 	float	rcv_loss_prob;
 };
 
-struct TestParms *	TestOpts;
+extern struct TestParms *	TestOpts;
 
 #define	TESTSEND	(TestOpts && TestOpts->enable_send_pkt_loss)
 #define	TESTRCV		(TestOpts && TestOpts->enable_rcv_pkt_loss)
 
-#define RandThresh(p) ((double)rand() <= ((((double)RAND_MAX) * ((double)p))))
+#ifdef  __GNUC__
+#define RandThresh(p) ((1.0*rand()) <= ((((double)RAND_MAX) * ((double)p))))
+#else
+#define RandThresh(p) ((double)(rand()) <= ((((double)RAND_MAX) * ((double)p))))
+#endif
 
 #define TestRand(field)	(TestOpts && RandThresh(TestOpts->field))
 int ParseTestOpts(void);
