@@ -1,3 +1,4 @@
+/* $Id: HBauth.h,v 1.2.2.1 2004/04/20 07:57:54 alan Exp $ */
 /*
  * auth.h: Authentication functions for Linux-HA
  *
@@ -30,8 +31,8 @@ struct HBauth_info {
 /* Authentication interfaces */
 struct HBAuthOps {
 	int (*auth)
-	(	const struct HBauth_info * authinfo, const char *data
-	,	char * result, int resultlen);
+	(	const struct HBauth_info * authinfo, const void *data
+	,	size_t data_len, char * result, int resultlen);
 	int		(*needskey) (void); 
 };
 
