@@ -1,4 +1,4 @@
-/* $Id: hb_api.h,v 1.12.2.1 2004/04/20 07:57:54 alan Exp $ */
+/* $Id: hb_api.h,v 1.12.2.3 2004/09/11 20:52:32 alan Exp $ */
 /*
  * Client-side Low-level clustering API for heartbeat.
  *
@@ -351,6 +351,10 @@ struct llc_ops {
 #define KEY_NORMALPOLL	"normalpoll"
 #define KEY_APIPERM	"apiauth"
 #define KEY_MSGFMT      "msgfmt"
+#define KEY_BADPACK	"log_badpack"
+#define KEY_REGAPPHBD	"use_apphbd"
+
+
 
 ll_cluster_t*	ll_cluster_new(const char * llctype);
 #endif /* __HB_API_H */
