@@ -1,4 +1,4 @@
-/* $Id: apphb.c,v 1.16.2.1 2004/04/20 09:08:50 lars Exp $ */
+/* $Id: apphb.c,v 1.16.2.2 2004/09/14 16:29:12 alan Exp $ */
 /*
  * apphb.c: application heartbeat library code.
  *
@@ -224,7 +224,7 @@ apphb_setwarn(unsigned long hbms)
 		errno = ESRCH;
 		return -1;
 	}
-	if (hbms < 0) {
+	if (hbms <= 0) {
 		errno = EINVAL;
 		return -1;
 	}
