@@ -1,4 +1,4 @@
-/* $Id: cl_msg.c,v 1.6.2.2 2004/04/29 01:23:44 alan Exp $ */
+/* $Id: cl_msg.c,v 1.6.2.9 2004/09/18 06:20:33 alan Exp $ */
 /*
  * Heartbeat messaging object.
  *
@@ -91,6 +91,8 @@ void ha_msg_audit(const struct ha_msg* msg);
 
 static volatile hb_msg_stats_t*	msgstats = NULL;
 
+gboolean cl_msg_quiet_fmterr = FALSE;
+
 extern int		netstring_format;
 
 static struct ha_msg* wirefmt2msg_ll(const char* s, size_t length, int need_auth);
@@ -171,6 +173,13 @@ convert(char* s, int len, int depth, int direction)
 }
 
 
+static int
+intlen(int x)
+{
+	char	buf[20];
+	return snprintf(buf, sizeof(buf), "%d", x);
+}
+
 
 /* Create a new (empty) message */
 struct ha_msg *
@@ -238,14 +247,15 @@ ha_msg_del(struct ha_msg *msg)
 		}
 		if (msg->values) {
 			for (j=0; j < msg->nfields; ++j) {
-				if (msg->values[j] && msg->types[j]
-				!=	FT_STRUCT) {
+				if (msg->values[j] == NULL) {
+					continue;
+				}
+				if (msg->types[j] != FT_STRUCT) {
 					ha_free(msg->values[j]);
-					msg->values[j] = NULL;
 				}else{
 					ha_msg_del((struct ha_msg*)msg->values[j]);
-					msg->values[j] = NULL;
 				}
+				msg->values[j] = NULL;
 			}
 			ha_free(msg->values);
 			msg->values = NULL;
@@ -277,7 +287,9 @@ ha_msg_copy(const struct ha_msg *msg)
 
 
 	AUDITMSG(msg);
-	ret = MALLOCT(struct ha_msg);
+	if (msg == NULL || (ret = MALLOCT(struct ha_msg)) == NULL) {
+		return NULL;
+	}
 	ret->nfields	= msg->nfields;
 	ret->nalloc	= msg->nalloc;
 	ret->stringlen	= msg->stringlen;
@@ -324,8 +336,12 @@ ha_msg_copy(const struct ha_msg *msg)
 	return ret;
 
 freeandleave:
-	ha_msg_del(ret);
-	ret=NULL;
+	/*
+	 * ha_msg_del nicely handles partially constructed ha_msgs
+	 * so, there's not really a memory leak here at all, but BEAM
+	 * thinks there is.
+	 */
+	ha_msg_del(ret);/* memory leak */	ret=NULL;
 	return ret;
 }
 
@@ -445,25 +461,30 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 	size_t	startlen_netstring = sizeof(MSG_START_NETSTRING) -1 ;
 	size_t	newstringlen;
 
-	char	*cp_name;
+	char	*cp_name = NULL;
 	size_t	cp_namelen;
 	size_t	cp_vallen;
-	char	*cp_value;
+	void	*cp_value = NULL;
 	int	internal_type;
 
 	if (!msg || (msg->nfields >= msg->nalloc)
-	    ||	msg->names == NULL || msg->values == NULL) {
-		cl_log(LOG_ERR, "ha_msg_addraw_ll: cannot add field to ha_msg");
+	||	msg->names == NULL || msg->values == NULL) {
+		cl_log(LOG_ERR
+		,	"ha_msg_addraw_ll: cannot add field to ha_msg");
 		return(HA_FAIL);
 	}
 
 	if (namelen >= startlen && strncmp(name, MSG_START, startlen) == 0) {
-		cl_log(LOG_ERR, "ha_msg_addraw_ll: illegal field");
+		if (!cl_msg_quiet_fmterr) {
+			cl_log(LOG_ERR, "ha_msg_addraw_ll: illegal field");
+		}
 		return(HA_FAIL);
 	}
 	if (namelen >= startlen_netstring
-	    && strncmp(name, MSG_START_NETSTRING, startlen_netstring == 0)){
-		cl_log(LOG_ERR, "ha_msg_addraw_ll: illegal field");
+	&&	strncmp(name, MSG_START_NETSTRING, startlen_netstring) == 0){
+		if (!cl_msg_quiet_fmterr) {
+			cl_log(LOG_ERR, "ha_msg_addraw_ll: illegal field");
+		}
 	}
 
 	if (name == NULL || value == NULL
@@ -484,6 +505,7 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 		cp_namelen = namelen;
 		cp_value = value;
 		cp_vallen = vallen;
+		internal_type = type;
 
 		break;
 	case FT_STRUCT:
@@ -499,22 +521,26 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 		msg->stringlen = newstringlen;
 		msg->netstringlen += 0;
 		/*  intlen(namelen) + (namelen) + intlen(vallen) + vallen + 4 */
-		msg->netstringlen += 0; //4; /* for type*/
+		msg->netstringlen += 0; /*4; */ /* for type*/
 		msg->types[next] = FT_STRUCT;
+		internal_type = type;
 
 		msg->nfields++;
 
 		return(HA_OK);
 
-	default: //case FT_STRING:
+	default: /*case FT_STRING:*/
 		newstringlen =  msg->stringlen + (namelen+vallen+2);
 
 		internal_type = FT_STRING;
 		if (name[0] == '('){
 
 			if (name[2] != ')'){
-				cl_log(LOG_ERR
-				, "ha_msg_addraw_ll(): no closing parentheses");
+				if (!cl_msg_quiet_fmterr) {
+					cl_log(LOG_ERR
+					, "ha_msg_addraw_ll()"
+					": no closing parentheses");
+				}
 				return(HA_FAIL);
 			}
 			sscanf(name + 1, "%d", &internal_type);
@@ -529,7 +555,7 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 
 		if (internal_type == FT_BINARY){
 			char	tmpbuf[MAXMSG];
-			int	nlo = 3; //name length overhead
+			int	nlo = 3; /*name length overhead*/
 
 			cp_name = name;
 			cp_namelen = namelen - nlo ;
@@ -543,7 +569,7 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 
 		}else if (internal_type ==  FT_STRUCT ){
 			struct ha_msg	*tmpmsg;
-			int	nlo = 3; //name length overhead
+			int	nlo = 3; /* iname length overhead */
 
 			cp_name = name;
 			cp_namelen = namelen - nlo ;
@@ -563,11 +589,17 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 				": string2msg_ll failed");
 				return(HA_FAIL);
 			}
-
-			cp_value = (void*) tmpmsg;
+			ha_free(value);
+			cp_value =  tmpmsg;
 			cp_vallen = sizeof(struct ha_msg);
 
 		}else{
+			/* SHOULDN'T THIS CASE ALSO BE namelen+nlo ??
+			 * FIXME!  (either this comment or the code)
+			 * And if it should be, then factor this code out
+			 * from all the branches of this if statement
+			 * At least explain why this is different.
+			 */
 			cp_name = name;
 			cp_namelen = namelen;		
 			cp_value = value;
@@ -580,6 +612,17 @@ ha_msg_addraw_ll(struct ha_msg * msg, char * name, size_t namelen,
 	if (newstringlen >= MAXMSG) {
 		cl_log(LOG_ERR, "ha_msg_addraw_ll(): "
 		       "cannot add name/value to ha_msg (value too big)");
+		if (cp_value) {
+			if (internal_type == FT_STRUCT) {
+				ha_msg_del((struct ha_msg *)cp_value);
+			}else{
+				cl_free(cp_value);
+			}
+			cp_value = NULL;
+		}
+		if (cp_name) {
+			cl_free(cp_name);	cp_name = NULL;
+		}
 		return(HA_FAIL);
 	}
 
@@ -732,8 +775,11 @@ ha_msg_add_nv_depth(struct ha_msg* msg, const char * nvline,
 	/* How many characters before the '='? */
 	if ((namelen = strcspn(nvline, EQUAL)) <= 0
 	||	nvline[namelen] != '=') {
-		cl_log(LOG_WARNING, "ha_msg_add_nv: line doesn't contain '='");
-		cl_log(LOG_INFO, "%s", nvline);
+		if (!cl_msg_quiet_fmterr) {
+			cl_log(LOG_WARNING
+			,	"ha_msg_add_nv: line doesn't contain '='");
+			cl_log(LOG_INFO, "%s", nvline);
+		}
 		return(HA_FAIL);
 	}
 	valp = nvline + namelen +1; /* Point just *past* the '=' */
@@ -851,7 +897,7 @@ cl_get_struct(const struct ha_msg *msg, const char* name)
 	ret = (struct ha_msg *)cl_get_value(msg, name, &vallen, &type);
 
 	if (ret == NULL || type != FT_STRUCT){
-		cl_log(LOG_ERR, "filed %s not found or"
+		cl_log(LOG_ERR, "field %s not found or"
 	       " it is not a struct", name);
 		return(NULL);
 	}
@@ -1008,9 +1054,11 @@ msgfromstream_netstring(FILE * f)
 		char *	typebuf;
 
 		if (fscanf(f, "%d:", &namelen) <= 0 || namelen <= 0){
-			cl_log(LOG_WARNING
-			,	" msgfromstream_netstring()"
-			": scanning for namelen failed");
+			if (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_WARNING
+				,	" msgfromstream_netstring()"
+				": scanning for namelen failed");
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1027,9 +1075,11 @@ msgfromstream_netstring(FILE * f)
 		}
 
 		if (*(namebuf + namelen) != ',' ){
-			cl_log(LOG_WARNING
-			,	"msgfromstream_netstring()"
-			": \",\" is missing in netstring for name");
+			if (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_WARNING
+				,	"msgfromstream_netstring()"
+				": \",\" is missing in netstring for name");
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1041,10 +1091,12 @@ msgfromstream_netstring(FILE * f)
 
 			if (!is_auth_netstring(total_databuf
 			,	sp - total_databuf, name,namelen) ){
-				cl_log(LOG_ERR
-				,	"msgfromstream_netstring()"
-				": netstring authentication"
-				" failed msgfromstream_netstring()");
+				if (!cl_msg_quiet_fmterr) {
+					cl_log(LOG_ERR
+					,	"msgfromstream_netstring()"
+					": netstring authentication"
+					" failed msgfromstream_netstring()");
+				}
 				cl_log_message(ret);
 				ha_msg_del(ret);
 				return(NULL);
@@ -1065,9 +1117,11 @@ msgfromstream_netstring(FILE * f)
 		}
 
 		if (*(typebuf + typelen) != ',' ){
-			cl_log(LOG_WARNING
-			,	"msgfromstream_netstring()"
-			": \",\" is missing in netstring for type");
+			if  (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_WARNING
+				,	"msgfromstream_netstring()"
+				": \",\" is missing in netstring for type");
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1076,9 +1130,11 @@ msgfromstream_netstring(FILE * f)
 		type = typebuf;
 
 		if (fscanf(f, "%d:", &datalen) <= 0) {
-			cl_log(LOG_WARNING
-			,	"msgfromstream_netstring()"
-			": scanning for datalen failed");
+			if (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_WARNING
+				,	"msgfromstream_netstring()"
+				": scanning for datalen failed");
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1096,9 +1152,11 @@ msgfromstream_netstring(FILE * f)
 		}
 
 		if (*(databuf + datalen ) != ',' ){
-			cl_log(LOG_WARNING
-			,	"msgfromstream_netstring()"
-			": \",\" is missing in netstring for data");
+			if (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_WARNING
+				,	"msgfromstream_netstring()"
+				": \",\" is missing in netstring for data");
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1403,7 +1461,9 @@ string2msg_ll(const char * s, size_t length, int depth, int need_auth)
 	if (strncmp(sp, MSG_START, startlen) != 0) {
 		/* This can happen if the sender gets killed */
 		/* at just the wrong time... */
-		cl_log(LOG_WARNING, "string2msg_ll: no MSG_START");
+		if (!cl_msg_quiet_fmterr) {
+			cl_log(LOG_WARNING, "string2msg_ll: no MSG_START");
+		}
 		ha_msg_del(ret);
 		return(NULL);
 	}else{
@@ -1427,8 +1487,10 @@ string2msg_ll(const char * s, size_t length, int depth, int need_auth)
 		}
 		/* Add the "name=value" string on this line to the message */
 		if (ha_msg_add_nv_depth(ret, sp, smax, depth) != HA_OK) {
-			cl_log(LOG_ERR, "NV failure (string2msg_ll):");
-			cl_log(LOG_ERR, "Input string: [%s]", s);
+			if (!cl_msg_quiet_fmterr) {
+				cl_log(LOG_ERR, "NV failure (string2msg_ll):");
+				cl_log(LOG_ERR, "Input string: [%s]", s);
+			}
 			ha_msg_del(ret);
 			return(NULL);
 		}
@@ -1441,9 +1503,11 @@ string2msg_ll(const char * s, size_t length, int depth, int need_auth)
 	if (need_auth && msg_authentication_method
 	&&		!msg_authentication_method(ret)) {
 		const char* from = ha_msg_value(ret, F_ORIG);
-		cl_log(LOG_WARNING
-	       ,       "string2msg_ll: node [%s]"
-	       " failed authentication", from ? from : "?");
+		if (!cl_msg_quiet_fmterr) {
+			cl_log(LOG_WARNING
+		       ,       "string2msg_ll: node [%s]"
+		       " failed authentication", from ? from : "?");
+		}
 		ha_msg_del(ret);
 		ret = NULL;
 	}
@@ -1690,9 +1754,11 @@ msg2wirefmt(const struct ha_msg*m, size_t* len)
 		char	*tmp;
 
 		tmp = msg2string(m);
-		//*len = m->stringlen;
-		//cl_log(LOG_INFO, "m->stringlen =%d,strlen(tmp)=%d",
-		//m->stringlen, strlen(tmp));
+/*
+		*len = m->stringlen;
+		cl_log(LOG_INFO, "m->stringlen =%d,strlen(tmp)=%d",
+		m->stringlen, strlen(tmp));
+*/
 		*len = strlen(tmp) + 1;
 		return(tmp);
 	}
@@ -1761,7 +1827,6 @@ cl_log_message (const struct ha_msg *m)
 	}
 }
 
-
 #ifdef TESTMAIN_MSGS
 int
 main(int argc, char ** argv)
@@ -1784,6 +1849,35 @@ main(int argc, char ** argv)
 #endif
 /*
  * $Log: cl_msg.c,v $
+ * Revision 1.6.2.9  2004/09/18 06:20:33  alan
+ * Changed a message on data corruption so that it won't come out
+ * if it's been suppressed.
+ *
+ * Revision 1.6.2.8  2004/09/15 06:27:58  alan
+ * BEAM FIX:  BEAM caught an subtle and extremely difficult to find bug.
+ * It noticed that in some branches a void* was from ha_malloc, and in other
+ * cases it was from ha_msg_new(), and that when I was freeing it, I could
+ * sometimes free the ha_msg with ha_malloc().  I fixed the code, and now
+ * it is quite happy.	AWESOME!  I'm impressed!
+ *
+ * Revision 1.6.2.7  2004/09/15 06:09:06  alan
+ * A few BEAM fixes - mainly for memory leaks -- and apparent ones ;-)
+ * This one includes a BEAM workaround that is works around what is
+ * IMHO a subtle BEAM bug...
+ *
+ * Revision 1.6.2.6  2004/09/14 18:15:48  alan
+ * BEAM FIX:  Two possible NULL pointer references eliminated.
+ *
+ * Revision 1.6.2.5  2004/09/11 06:36:33  msoffen
+ * Moved variable  definitions to top of functions/blocks, changed comments to c style comments.
+ *
+ * Revision 1.6.2.4  2004/08/31 22:29:19  alan
+ * Fixed a memory leak (brought back from HEAD branch)
+ *
+ * Revision 1.6.2.3  2004/08/05 18:30:47  alan
+ * Put in changes to support a new option which allows one to suppress
+ * notification of packet corruption.
+ *
  * Revision 1.6.2.2  2004/04/29 01:23:44  alan
  * Repaired a completely broken fix to a slight portability problem for FreeBSD.
  * %xd is NOT remotely equivalent to %zd.
