@@ -1,4 +1,4 @@
-static const char _mcast_Id [] = "$Id: mcast.c,v 1.14 2003/02/07 08:37:17 horms Exp $";
+/* $Id: mcast.c,v 1.15.2.3 2004/05/12 00:10:14 alan Exp $ */
 /*
  * mcast.c: implements hearbeat API for UDP multicast communication
  *
@@ -70,8 +70,8 @@ static struct hb_media * mcast_new(const char * intf, const char *mcast
 			,	u_short port, u_char ttl, u_char loop);
 static int		mcast_open(struct hb_media* mp);
 static int		mcast_close(struct hb_media* mp);
-static struct ha_msg*	mcast_read(struct hb_media* mp);
-static int		mcast_write(struct hb_media* mp, struct ha_msg* msg);
+static void*		mcast_read(struct hb_media* mp, int* lenp);
+static int		mcast_write(struct hb_media* mp, void* p, int len);
 static int		mcast_descr(char** buffer);
 static int		mcast_mtype(char** buffer);
 static int		mcast_isping(void);
@@ -123,6 +123,7 @@ static void*			interfprivate;
 
 #define LOG	PluginImports->log
 #define MALLOC	PluginImports->alloc
+#define STRDUP  PluginImports->mstrdup
 #define FREE	PluginImports->mfree
 
 PIL_rc
@@ -133,9 +134,6 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
 {
 	/* Force the compiler to do a little type checking */
 	(void)(PILPluginInitFun)PIL_PLUGIN_INIT;
-	(void)_heartbeat_h_Id;
-	(void)_ha_msg_h_Id;
-	(void)_mcast_Id;
 
 	PluginImports = imports;
 	OurPlugin = us;
@@ -178,25 +176,23 @@ static int get_loop(const char *loop, u_char *l);
 static int
 mcast_mtype(char** buffer)
 { 
-	
-	*buffer = MALLOC(sizeof(PIL_PLUGIN_S));
+	*buffer = STRDUP(PIL_PLUGIN_S);
+	if (!*buffer) {
+		return 0;
+	}
 
-	strcpy(*buffer, PIL_PLUGIN_S);
-
-	return STRLEN(PIL_PLUGIN_S);
+	return STRLEN(*buffer);
 }
 
 static int
 mcast_descr(char **buffer)
 { 
+	*buffer = STRDUP("UDP/IP multicast");
+	if (!*buffer) {
+		return 0;
+	}
 
-	const char str[] = "UDP/IP multicast";	
-
-	*buffer = MALLOC(sizeof(str));
-
-	strcpy(*buffer, str);
-
-	return STRLEN(str);
+	return STRLEN(*buffer);
 }
 
 static int
@@ -239,7 +235,7 @@ mcast_parse(const char *line)
 
 	if (*dev != EOS)  {
 		if (!is_valid_dev(dev)) {
-			LOG(PIL_CRIT, "mcast bad device [%s]", dev);
+			PILCallLog(LOG, PIL_CRIT, "mcast bad device [%s]", dev);
 			return HA_FAIL;
 		}
 		/* Skip over white space, then grab the multicast group */
@@ -250,12 +246,12 @@ mcast_parse(const char *line)
 		mcast[toklen] = EOS;
 	
 		if (*mcast == EOS)  {
-			LOG(PIL_CRIT, "mcast [%s] missing mcast address",
+			PILCallLog(LOG, PIL_CRIT, "mcast [%s] missing mcast address",
 				dev);
 			return(HA_FAIL);
 		}
 		if (!is_valid_mcast_addr(mcast)) {
-			LOG(PIL_CRIT, "mcast [%s] bad addr [%s]", dev, mcast);
+			PILCallLog(LOG, PIL_CRIT, "mcast [%s] bad addr [%s]", dev, mcast);
 			return(HA_FAIL);
 		}
 
@@ -267,11 +263,11 @@ mcast_parse(const char *line)
 		token[toklen] = EOS;
 
 		if (*token == EOS)  {
-			LOG(PIL_CRIT, "mcast [%s] missing port", dev);
+			PILCallLog(LOG, PIL_CRIT, "mcast [%s] missing port", dev);
 			return(HA_FAIL);
 		}
 		if (get_port(token, &port) == -1) {
-			LOG(PIL_CRIT, " mcast [%s] bad port [%s]", dev, port);
+			PILCallLog(LOG, PIL_CRIT, " mcast [%s] bad port [%d]", dev, port);
 			return HA_FAIL;
 		}
 
@@ -283,11 +279,11 @@ mcast_parse(const char *line)
 		token[toklen] = EOS;
 
 		if (*token == EOS)  {
-			LOG(PIL_CRIT, "mcast [%s] missing ttl", dev);
+			PILCallLog(LOG, PIL_CRIT, "mcast [%s] missing ttl", dev);
 			return(HA_FAIL);
 		}
 		if (get_ttl(token, &ttl) == -1) {
-			LOG(PIL_CRIT, " mcast [%s] bad ttl [%s]", dev, ttl);
+			PILCallLog(LOG, PIL_CRIT, " mcast [%s] bad ttl [%d]", dev, ttl);
 			return HA_FAIL;
 		}
 
@@ -299,11 +295,11 @@ mcast_parse(const char *line)
 		token[toklen] = EOS;
 
 		if (*token == EOS)  {
-			LOG(PIL_CRIT, "mcast [%s] missing loop", dev);
+			PILCallLog(LOG, PIL_CRIT, "mcast [%s] missing loop", dev);
 			return(HA_FAIL);
 		}
 		if (get_loop(token, &loop) == -1) {
-			LOG(PIL_CRIT, " mcast [%s] bad loop [%s]", dev, loop);
+			PILCallLog(LOG, PIL_CRIT, " mcast [%s] bad loop [%d]", dev, loop);
 			return HA_FAIL;
 		}
 
@@ -332,7 +328,7 @@ mcast_new(const char * intf, const char *mcast, u_short port,
 	/* create new mcast_private struct...hmmm...who frees it? */
 	mcp = new_mcast_private(intf, mcast, port, ttl, loop);
 	if (mcp == NULL) {
-		LOG(PIL_WARN, "Error creating mcast_private(%s, %s, %d, %d, %d)",
+		PILCallLog(LOG, PIL_WARN, "Error creating mcast_private(%s, %s, %d, %d, %d)",
 			 intf, mcast, port, ttl, loop);
 		return(NULL);
 	}
@@ -340,11 +336,17 @@ mcast_new(const char * intf, const char *mcast, u_short port,
 	if (ret != NULL) {
 		char * name;
 		ret->pd = (void*)mcp;
-		name = MALLOC(strlen(intf)+1);
-		strcpy(name, intf);
-		ret->name = name;
+		name = STRDUP(intf);
+		if (name != NULL) {
+			ret->name = name;
+		}
+		else {
+			FREE(ret);
+			ret = NULL;
+		}
 
-	}else{
+	}
+	if(ret == NULL) {
 		FREE(mcp->interface);
 		FREE(mcp);
 	}
@@ -370,7 +372,7 @@ mcast_open(struct hb_media* hbm)
 		return(HA_FAIL);
 	}
 
-	LOG(PIL_INFO, "UDP multicast heartbeat started for group %s "
+	PILCallLog(LOG, PIL_INFO, "UDP multicast heartbeat started for group %s "
 		"port %d interface %s (ttl=%d loop=%d)" , inet_ntoa(mcp->mcast),
 		mcp->port, mcp->interface, mcp->ttl, mcp->loop);
 
@@ -401,41 +403,56 @@ mcast_close(struct hb_media* hbm)
 	}
 	return(rc);
 }
+
 /*
  * Receive a heartbeat multicast packet from UDP interface
  */
 
-static struct ha_msg *
-mcast_read(struct hb_media* hbm)
+static void *
+mcast_read(struct hb_media* hbm, int *lenp)
 {
 	struct mcast_private *	mcp;
 	char			buf[MAXLINE];
 	int			addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	int	numbytes;
+	void	*pkt;
 
 	MCASTASSERT(hbm);
 	mcp = (struct mcast_private *) hbm->pd;
-
+	
 	if ((numbytes=recvfrom(mcp->rsocket, buf, MAXLINE-1, 0
-	,	(struct sockaddr *)&their_addr, &addr_len)) < 0) {
+			       ,(struct sockaddr *)&their_addr, &addr_len)) < 0) {
 		if (errno != EINTR) {
-			LOG(PIL_CRIT, "Error receiving from socket: %s"
-			,	strerror(errno));
+			PILCallLog(LOG, PIL_CRIT, "Error receiving from socket: %s"
+			    ,	strerror(errno));
 		}
 		return NULL;
 	}
 	/* Avoid possible buffer overruns */
 	buf[numbytes] = EOS;
-
+	
 	if (Debug >= PKTTRACE) {
-		LOG(PIL_DEBUG, "got %d byte packet from %s"
-		,	numbytes, inet_ntoa(their_addr.sin_addr));
+		PILCallLog(LOG, PIL_DEBUG, "got %d byte packet from %s"
+		    ,	numbytes, inet_ntoa(their_addr.sin_addr));
 	}
 	if (Debug >= PKTCONTTRACE && numbytes > 0) {
-		LOG(PIL_DEBUG, buf);
+		PILCallLog(LOG, PIL_DEBUG, "%s", buf);
 	}
-	return(string2msg(buf, sizeof(buf)));
+	
+	pkt = ha_malloc(numbytes + 1);
+	if(!pkt){
+		PILCallLog(LOG, PIL_CRIT
+		    ,	"Error in allocating memory");
+		return(NULL);
+	}
+	
+	memcpy(pkt, buf, numbytes + 1);
+	*lenp = numbytes + 1 ;
+
+	return(pkt);
+	
+	
 }
 
 /*
@@ -443,38 +460,34 @@ mcast_read(struct hb_media* hbm)
  */
 
 static int
-mcast_write(struct hb_media* hbm, struct ha_msg * msgptr)
+mcast_write(struct hb_media* hbm, void *pkt, int len)
 {
 	struct mcast_private *	mcp;
 	int			rc;
-	char*			pkt;
-	int			size;
-
+	
 	MCASTASSERT(hbm);
 	mcp = (struct mcast_private *) hbm->pd;
 
-	if ((pkt = msg2string(msgptr)) == NULL)  {
-		return(HA_FAIL);
-	}
-	size = strlen(pkt)+1;
-
-	if ((rc=sendto(mcp->wsocket, pkt, size, 0
+	if ((rc=sendto(mcp->wsocket, pkt, len, 0
 	,	(struct sockaddr *)&mcp->addr
-	,	sizeof(struct sockaddr))) != size) {
-		LOG(PIL_WARN, "Error sending packet: %s", strerror(errno));
-		ha_free(pkt);
+	,	sizeof(struct sockaddr))) != len) {
+		PILCallLog(LOG, PIL_CRIT, "Unable to send mcast packet [%d]: %s"
+		,	rc, strerror(errno));
 		return(HA_FAIL);
 	}
-
+	
 	if (Debug >= PKTTRACE) {
-		LOG(PIL_DEBUG, "sent %d bytes to %s"
-		,	rc, inet_ntoa(mcp->addr.sin_addr));
+		PILCallLog(LOG, PIL_DEBUG, "sent %d bytes to %s"
+		    ,	rc, inet_ntoa(mcp->addr.sin_addr));
    	}
 	if (Debug >= PKTCONTTRACE) {
-		LOG(PIL_DEBUG, pkt);
+		PILCallLog(LOG, PIL_DEBUG, "%s", (const char *)pkt);
    	}
-	ha_free(pkt);
 	return(HA_OK);
+
+
+  return(HA_OK);
+  
 }
 
 /*
@@ -490,24 +503,24 @@ mcast_make_send_sock(struct hb_media * hbm)
 	mcp = (struct mcast_private *) hbm->pd;
 
 	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-		LOG(PIL_WARN, "Error getting socket: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error getting socket: %s", strerror(errno));
 		return(sockfd);
    	}
 
 	if (set_mcast_if(sockfd, mcp->interface) < 0) {
-		LOG(PIL_WARN, "Error setting outbound mcast interface: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error setting outbound mcast interface: %s", strerror(errno));
 	}
 
 	if (set_mcast_loop(sockfd, mcp->loop) < 0) {
-		LOG(PIL_WARN, "Error setting outbound mcast loopback value: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error setting outbound mcast loopback value: %s", strerror(errno));
 	}
 
 	if (set_mcast_ttl(sockfd, mcp->ttl) < 0) {
-		LOG(PIL_WARN, "Error setting outbound mcast TTL: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error setting outbound mcast TTL: %s", strerror(errno));
 	}
 
 	if (fcntl(sockfd,F_SETFD, FD_CLOEXEC)) {
-		LOG(PIL_WARN, "Error setting the close-on-exec flag: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error setting the close-on-exec flag: %s", strerror(errno));
 	}
 	return(sockfd);
 }
@@ -533,13 +546,13 @@ mcast_make_receive_sock(struct hb_media * hbm)
 	mcp = (struct mcast_private *) hbm->pd;
 
 	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
-		LOG(PIL_CRIT, "Error getting socket");
+		PILCallLog(LOG, PIL_CRIT, "Error getting socket");
 		return -1;
 	}
 	/* set REUSEADDR option on socket so you can bind a multicast */
 	/* reader to multiple interfaces */
 	if(setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (void *)&one, sizeof(one)) < 0){
-		LOG(PIL_CRIT, "Error setsockopt(SO_REUSEADDR)");
+		PILCallLog(LOG, PIL_CRIT, "Error setsockopt(SO_REUSEADDR)");
 	}        
 
 	/* ripped off from udp.c, if we all use SO_REUSEADDR */
@@ -554,7 +567,7 @@ mcast_make_receive_sock(struct hb_media * hbm)
 			boundyet=1;
 		} else if (rc == -1) {
 			if (binderr == EADDRINUSE) {
-				LOG(PIL_CRIT, "Can't bind (EADDRINUSE), "
+				PILCallLog(LOG, PIL_CRIT, "Can't bind (EADDRINUSE), "
 					"retrying");
 				sleep(1);
 			} else	{ 
@@ -567,15 +580,15 @@ mcast_make_receive_sock(struct hb_media * hbm)
 	if (!boundyet) {
 		if (binderr == EADDRINUSE) {
 			/* This happens with multiple udp or ppp interfaces */
-			LOG(PIL_INFO
+			PILCallLog(LOG, PIL_INFO
 			,	"Someone already listening on port %d [%s]"
 			,	mcp->port
 			,	mcp->interface);
-			LOG(PIL_INFO, "multicast read process exiting");
+			PILCallLog(LOG, PIL_INFO, "multicast read process exiting");
 			close(sockfd);
 			cleanexit(0);
 		} else {
-			LOG(PIL_WARN, "Unable to bind socket. Giving up: %s", strerror(errno));
+			PILCallLog(LOG, PIL_WARN, "Unable to bind socket. Giving up: %s", strerror(errno));
 			close(sockfd);
 			return(-1);
 		}
@@ -583,18 +596,18 @@ mcast_make_receive_sock(struct hb_media * hbm)
 	/* join the multicast group...this is what really makes this a */
 	/* multicast reader */
 	if (join_mcast_group(sockfd, &mcp->mcast, mcp->interface) == -1) {
-		LOG(PIL_CRIT, "Can't join multicast group %s on interface %s",
+		PILCallLog(LOG, PIL_CRIT, "Can't join multicast group %s on interface %s",
 			inet_ntoa(mcp->mcast), mcp->interface);
-		LOG(PIL_INFO, "multicast read process exiting");
+		PILCallLog(LOG, PIL_INFO, "multicast read process exiting");
 		close(sockfd);
 		cleanexit(0);
 	}
 	if (ANYDEBUG) 
-		LOG(PIL_DEBUG, "Successfully joined multicast group %s on"
+		PILCallLog(LOG, PIL_DEBUG, "Successfully joined multicast group %s on"
 			"interface %s", inet_ntoa(mcp->mcast), mcp->interface);
 		
 	if (fcntl(sockfd,F_SETFD, FD_CLOEXEC)) {
-		LOG(PIL_WARN, "Error setting the close-on-exec flag: %s", strerror(errno));
+		PILCallLog(LOG, PIL_WARN, "Error setting the close-on-exec flag: %s", strerror(errno));
 	}
 	return(sockfd);
 }
@@ -610,12 +623,11 @@ new_mcast_private(const char *ifn, const char *mcast, u_short port,
 		return NULL;
 	}
 
-	mcp->interface = (char *)MALLOC(strlen(ifn)+1);
+	mcp->interface = (char *)STRDUP(ifn);
 	if(mcp->interface == NULL) {
 		FREE(mcp);
 		return NULL;
 	}
-	strcpy(mcp->interface, ifn);
 
 	/* Set up multicast address */
 
@@ -734,15 +746,15 @@ if_getaddr(const char *ifname, struct in_addr *addr)
 	}
 
 	if ((fd=socket(AF_INET, SOCK_DGRAM, 0)) == -1)	{
-		LOG(PIL_CRIT, "Error getting socket");
+		PILCallLog(LOG, PIL_CRIT, "Error getting socket");
 		return -1;
 	}
 	if (Debug > 0) {
-		LOG(PIL_DEBUG,	"looking up address for %s"
+		PILCallLog(LOG, PIL_DEBUG,	"looking up address for %s"
 		,	if_info.ifr_name);
 	}
 	if (ioctl(fd, SIOCGIFADDR, &if_info) == -1) {
-		LOG(PIL_CRIT, "Error ioctl(SIOCGIFADDR)");
+		PILCallLog(LOG, PIL_CRIT, "Error ioctl(SIOCGIFADDR)");
 		close(fd);
 		return -1;
 	}
@@ -819,6 +831,37 @@ get_loop(const char *loop, u_char *l)
 
 /*
  * $Log: mcast.c,v $
+ * Revision 1.15.2.3  2004/05/12 00:10:14  alan
+ * Changed all the HBcomm plugins to use PILCallLog() for logging
+ * instead of calling the function pointer directly.
+ * Also, in the process fixed several mismatches between arguments
+ * and format strings, and a couple of format string vulnerabilities.
+ *
+ * Revision 1.19  2004/05/11 22:04:35  alan
+ * Changed all the HBcomm plugins to use PILCallLog() for logging instead of calling
+ * the function pointer directly.
+ * Also, in the process fixed several mismatches between arguments and format strings, and
+ * a couple of format string vulnerabilities.
+ *
+ * Revision 1.18  2004/04/28 22:30:29  alan
+ * Put in some fixes for extra freeing of memory in the communications plugins.
+ *
+ * Revision 1.17  2004/03/03 05:31:50  alan
+ * Put in Gochun Shi's new netstrings on-the-wire data format code.
+ * this allows sending binary data, among many other things!
+ *
+ * Revision 1.16  2004/02/17 22:11:59  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.15  2004/01/21 11:34:15  horms
+ * - Replaced numerous malloc + strcpy/strncpy invocations with strdup
+ *   * This usually makes the code a bit cleaner
+ *   * Also is easier not to make code with potential buffer over-runs
+ * - Added STRDUP to pils modules
+ * - Removed some spurious MALLOC and FREE redefinitions
+ *   _that could never be used_
+ * - Make sure the return value of strdup is honoured in error conditions
+ *
  * Revision 1.14  2003/02/07 08:37:17  horms
  * Removed inclusion of portability.h from .h files
  * so that it does not need to be installed.
