@@ -1,4 +1,4 @@
-/* $Id: ping.c,v 1.26.2.2 2004/05/12 00:10:14 alan Exp $ */
+/* $Id: ping.c,v 1.26.2.3 2004/09/14 16:35:32 alan Exp $ */
 /*
  * ping.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -379,7 +379,9 @@ ping_read(struct hb_media* mp, int *lenp)
 	}
 	
 	pktlen = numbytes - hlen - ICMP_HDR_SZ;
-	pkt = ha_malloc(pktlen + 1);
+	if ((pkt = ha_malloc(pktlen + 1)) == NULL) {
+		return NULL;
+	}
 	pkt[pktlen] = 0;
 	
 	memcpy(pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);	
