@@ -1,4 +1,4 @@
-static const char _ucast_Id [] = "$Id: ucast.c,v 1.15 2003/02/07 08:37:18 horms Exp $";
+/* $Id: ucast.c,v 1.17.2.3 2004/05/12 00:10:14 alan Exp $ */
 /*
  * Adapted from alanr's UDP broadcast heartbeat bcast.c by Stphane Billiart
  *	<stephane@reefedge.com>
@@ -79,6 +79,7 @@ static const char _ucast_Id [] = "$Id: ucast.c,v 1.15 2003/02/07 08:37:18 horms
 
 #define LOG		PluginImports->log
 #define MALLOC		PluginImports->alloc
+#define STRDUP  	PluginImports->mstrdup
 #define FREE		PluginImports->mfree
 
 #define	MAXBINDTRIES	10
@@ -110,8 +111,8 @@ static int ucast_parse(const char *line);
 static struct hb_media* ucast_new(const char *intf, const char *addr);
 static int ucast_open(struct hb_media *mp);
 static int ucast_close(struct hb_media *mp);
-static struct ha_msg* ucast_read(struct hb_media *mp);
-static int ucast_write(struct hb_media *mp, struct ha_msg *msg);
+static void* ucast_read(struct hb_media *mp, int* lenp);
+static int ucast_write(struct hb_media *mp, void *msg, int len);
 
 static int HB_make_receive_sock(struct hb_media *ei);
 static int HB_make_send_sock(struct hb_media *mp);
@@ -207,7 +208,7 @@ static int ucast_parse(const char *line)
 	if (*dev != EOS)  {
 #ifdef NOTYET
 		if (!is_valid_dev(dev)) {
-			LOG(PIL_CRIT, "ucast: bad device [%s]", dev);
+			PILCallLog(LOG, PIL_CRIT, "ucast: bad device [%s]", dev);
 			return HA_FAIL;
 		}
 #endif
@@ -218,7 +219,7 @@ static int ucast_parse(const char *line)
 		ucast[toklen] = EOS;
 	
 		if (*ucast == EOS)  {
-			LOG(PIL_CRIT,
+			PILCallLog(LOG, PIL_CRIT,
 			  "ucast: [%s] missing target IP address/hostname",
 			  dev);
 			return HA_FAIL;
@@ -234,34 +235,26 @@ static int ucast_parse(const char *line)
 
 static int ucast_mtype(char **buffer)
 {
-	char *p;
-	int len;
-
-	len = strlen(PIL_PLUGIN_S);
-	if (!(p  = MALLOC(len * sizeof(char) + 1))) {
-		LOG(PIL_CRIT, "ucast: memory allocation error (line %d)",
-			(__LINE__ - 2) );
+	*buffer = STRDUP(PIL_PLUGIN_S);
+	if (!*buffer) {
+		PILCallLog(LOG, PIL_CRIT, "ucast: memory allocation error (line %d)",
+				(__LINE__ - 2) );
 		return 0;
 	}
-	strcpy(p, PIL_PLUGIN_S);
-	*buffer = p;
-	return len;
+
+	return strlen(*buffer);
 }
 
 static int ucast_descr(char **buffer)
 { 
-	char *p;
-	int len;
-
-	len = strlen(hb_media_name);
-	if (!(p = MALLOC(len * sizeof(char) + 1))) {
-		LOG(PIL_CRIT, "ucast: memory allocation error (line %d)",
-			(__LINE__ - 2) );
+	*buffer = strdup(hb_media_name);
+	if (!*buffer) {
+		PILCallLog(LOG, PIL_CRIT, "ucast: memory allocation error (line %d)",
+				(__LINE__ - 2) );
 		return 0;
 	}
-	strcpy(p, hb_media_name);
-	*buffer = p;
-	return len;
+
+	return strlen(*buffer);
 }
 
 static int ucast_isping(void)
@@ -273,10 +266,6 @@ static int ucast_init(void)
 {
 	struct servent *service;
 
-	(void)_heartbeat_h_Id;
-	(void)_ha_msg_h_Id;
-	(void)_ucast_Id;
-
 	g_assert(OurImports != NULL);
 
 	if (localudpport <= 0) {
@@ -307,7 +296,8 @@ static int ucast_init(void)
  *	Create new UDP/IP unicast heartbeat object 
  *	Name of interface and address are passed as parameters
  */
-static struct hb_media* ucast_new(const char *intf, const char *addr)
+static struct hb_media*
+ucast_new(const char *intf, const char *addr)
 {
 	struct ip_private *ipi;
 	struct hb_media *ret;
@@ -316,19 +306,19 @@ static struct hb_media* ucast_new(const char *intf, const char *addr)
 	ucast_init();
 
 	if (!(ipi = new_ip_interface(intf, addr, localudpport))) {
-		LOG(PIL_CRIT, "ucast: interface [%s] does not exist", intf);
+		PILCallLog(LOG, PIL_CRIT, "ucast: interface [%s] does not exist", intf);
 		return NULL;
 	}
 	if (!(ret = (struct hb_media*)MALLOC(sizeof(struct hb_media)))) {
-		LOG(PIL_CRIT, "ucast: memory allocation error (line %d)",
+		PILCallLog(LOG, PIL_CRIT, "ucast: memory allocation error (line %d)",
 			(__LINE__ - 2) );
 		FREE(ipi->interface);
 		FREE(ipi);
 	}
 	else {
 		ret->pd = (void*)ipi;
-		if (!(name = MALLOC(strlen(intf)+1))) {
-			LOG(PIL_CRIT,
+		if (!(name = STRDUP(intf))) {
+			PILCallLog(LOG, PIL_CRIT,
 				"ucast: memory allocation error (line %d)",
 				(__LINE__ - 3) );
 			FREE(ipi->interface);
@@ -337,7 +327,6 @@ static struct hb_media* ucast_new(const char *intf, const char *addr)
 			ret = NULL;
 		}
 		else {
-			strcpy(name, intf);
 			ret->name = name;
 		}
 	}
@@ -362,7 +351,7 @@ static int ucast_open(struct hb_media* mp)
 		return HA_FAIL;
 	}
 
-	LOG(PIL_INFO, "ucast: started on port %d interface %s to %s",
+	PILCallLog(LOG, PIL_INFO, "ucast: started on port %d interface %s to %s",
 		localudpport, ei->interface, inet_ntoa(ei->addr.sin_addr));
 
 	return HA_OK;
@@ -390,17 +379,21 @@ static int ucast_close(struct hb_media* mp)
 	return rc;
 }
 
+
 /*
  * Receive a heartbeat unicast packet from UDP interface
  */
-static struct ha_msg* ucast_read(struct hb_media *mp)
+
+static void *
+ucast_read(struct hb_media* mp, int *lenp)
 {
 	struct ip_private *ei;
 	int addr_len;
 	struct sockaddr_in their_addr;
 	int numbytes;
 	char buf[MAXLINE];
-
+	void	*pkt;
+	
 	UCASTASSERT(mp);
 	ei = (struct ip_private*)mp->pd;
 
@@ -408,66 +401,71 @@ static struct ha_msg* ucast_read(struct hb_media *mp)
 	if ((numbytes = recvfrom(ei->rsocket, buf, MAXLINE-1, 0,
 		(struct sockaddr *)&their_addr, &addr_len)) == -1) {
 		if (errno != EINTR) {
-			LOG(PIL_CRIT, "ucast: error receiving from socket: %s",
+			PILCallLog(LOG, PIL_CRIT, "ucast: error receiving from socket: %s",
 				strerror(errno));
 		}
 		return NULL;
 	}
 	if (numbytes == 0) {
-		LOG(PIL_CRIT, "ucast: received zero bytes");
+		PILCallLog(LOG, PIL_CRIT, "ucast: received zero bytes");
 		return NULL;
 	}
 
 	buf[numbytes] = EOS;
 
 	if (DEBUGPKT) {
-		LOG(LOG_DEBUG, "ucast: received %d byte packet from %s",
+		PILCallLog(LOG, PIL_DEBUG, "ucast: received %d byte packet from %s",
 			numbytes, inet_ntoa(their_addr.sin_addr));
 	}
 	if (DEBUGPKTCONT) {
-		LOG(LOG_DEBUG, buf);
+		PILCallLog(LOG, PIL_DEBUG, "%s", buf);
 	}
-	return string2msg(buf, sizeof(buf));
+
+		
+	pkt = ha_malloc(numbytes + 1);
+	if(!pkt){
+		PILCallLog(LOG, PIL_CRIT
+			   ,	"Error in allocating memory");
+		return(NULL);
+	}
+	
+	memcpy(pkt, buf, numbytes + 1);		
+	*lenp = numbytes +1;
+	
+	return(pkt);	
+	
+	
 }
 
 /*
  * Send a heartbeat packet over unicast UDP/IP interface
  */
-static int ucast_write(struct hb_media *mp, struct ha_msg *msgptr)
+
+static int
+ucast_write(struct hb_media* mp, void *pkt, int len)
 {
 	struct ip_private *ei;
 	int rc;
-	char *pkt;
-	int size;
-
+	
 	UCASTASSERT(mp);
 	ei = (struct ip_private*)mp->pd;
-
-	if (!(pkt = msg2string(msgptr)))
-		return HA_FAIL;
-
-	size = strlen(pkt)+1;
-
-	if ((rc = sendto(ei->wsocket, pkt, size, 0,
-			(struct sockaddr *)&ei->addr,
-			sizeof(struct sockaddr))) != size) {
-		LOG(PIL_CRIT, "ucast: error sending packet: %s",
-			strerror(errno));
-		ha_free(pkt);
+	
+	if ((rc = sendto(ei->wsocket, pkt, len, 0
+	,		(struct sockaddr *)&ei->addr
+	,		 sizeof(struct sockaddr))) != len) {
+		PILCallLog(LOG, PIL_CRIT, "Unable to send [%d] ucast packet: %s"
+		,	rc, strerror(errno));
 		return HA_FAIL;
 	}
-
+	
 	if (DEBUGPKT) {
-		LOG(LOG_DEBUG, "ucast: sent %d bytes to %s", rc,
-			inet_ntoa(ei->addr.sin_addr));
+		PILCallLog(LOG, PIL_DEBUG, "ucast: sent %d bytes to %s", rc,
+		    inet_ntoa(ei->addr.sin_addr));
    	}
 	if (DEBUGPKTCONT) {
-		LOG(LOG_DEBUG, pkt);
+		PILCallLog(LOG, PIL_DEBUG, "%s", (const char*)pkt);
    	}
-
-	ha_free(pkt);
-
-	return HA_OK;
+	return HA_OK;	
 }
 
 /*
@@ -487,7 +485,7 @@ static int HB_make_send_sock(struct hb_media *mp)
 	ei = (struct ip_private*)mp->pd;
 
 	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-		LOG(PIL_CRIT, "ucast: Error creating write socket: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: Error creating write socket: %s",
 			strerror(errno));
    	}
 
@@ -498,11 +496,11 @@ static int HB_make_send_sock(struct hb_media *mp)
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(sockfd, IPPROTO_IP, IP_TOS,
 				&tos, sizeof(tos)) < 0) {
-		LOG(PIL_CRIT, "ucast: error setting socket option IP_TOS: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: error setting socket option IP_TOS: %s",
 					strerror(errno));
 	}
 	else {
-		LOG(PIL_INFO,
+		PILCallLog(LOG, PIL_INFO,
 		  "ucast: write socket priority set to IPTOS_LOWDELAY on %s",
 		  ei->interface);
 	}
@@ -518,18 +516,18 @@ static int HB_make_send_sock(struct hb_media *mp)
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_BINDTODEVICE,
 				&i, sizeof(i)) == -1) {
-			LOG(PIL_CRIT,
+			PILCallLog(LOG, PIL_CRIT,
 			  "ucast: error setting option SO_BINDTODEVICE(w) on %s: %s",
 			  i.ifr_name, strerror(errno));
 			close(sockfd);
 			return -1;
 		}
-		LOG(PIL_INFO, "ucast: bound send socket to device: %s",
+		PILCallLog(LOG, PIL_INFO, "ucast: bound send socket to device: %s",
 			i.ifr_name);
 	}
 #endif
 	if (fcntl(sockfd,F_SETFD, FD_CLOEXEC) < 0) {
-		LOG(PIL_CRIT, "ucast: error setting close-on-exec flag: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: error setting close-on-exec flag: %s",
 			strerror(errno));
 	}
 
@@ -558,7 +556,7 @@ static int HB_make_receive_sock(struct hb_media *mp) {
 	my_addr.sin_addr.s_addr = INADDR_ANY;	/* auto-fill with my IP */
 
 	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
-		LOG(PIL_CRIT, "ucast: error creating read socket: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: error creating read socket: %s",
 			strerror(errno));
 		return -1;
 	}
@@ -574,7 +572,7 @@ static int HB_make_receive_sock(struct hb_media *mp) {
 	if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR,
 			(void *)&j, sizeof j) < 0) {
 		/* Ignore it.  It will almost always be OK anyway. */
-		LOG(PIL_CRIT,
+		PILCallLog(LOG, PIL_CRIT,
 			"ucast: error setting socket option SO_REUSEADDR: %s",
 			strerror(errno));
 	}        
@@ -588,13 +586,13 @@ static int HB_make_receive_sock(struct hb_media *mp) {
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_BINDTODEVICE,
 				&i, sizeof(i)) == -1) {
-			LOG(PIL_CRIT,
+			PILCallLog(LOG, PIL_CRIT,
 			  "ucast: error setting option SO_BINDTODEVICE(r) on %s: %s",
 			  i.ifr_name, strerror(errno));
 			close(sockfd);
 			return -1;
 		}
-		LOG(PIL_INFO, "ucast: bound receive socket to device: %s",
+		PILCallLog(LOG, PIL_INFO, "ucast: bound receive socket to device: %s",
 			i.ifr_name);
 	}
 #endif
@@ -605,7 +603,7 @@ static int HB_make_receive_sock(struct hb_media *mp) {
 	for (bindtries=0; !boundyet && bindtries < MAXBINDTRIES; ++bindtries) {
 		if (bind(sockfd, (struct sockaddr *)&my_addr,
 				sizeof(struct sockaddr)) < 0) {
-			LOG(PIL_CRIT, "ucast: error binding socket. Retrying: %s",
+			PILCallLog(LOG, PIL_CRIT, "ucast: error binding socket. Retrying: %s",
 				strerror(errno));
 			sleep(1);
 		}
@@ -617,22 +615,22 @@ static int HB_make_receive_sock(struct hb_media *mp) {
 #if !defined(SO_BINDTODEVICE)
 		if (errno == EADDRINUSE) {
 			/* This happens with multiple udp or ppp interfaces */
-			LOG(PIL_INFO,
+			PILCallLog(LOG, PIL_INFO,
 			  "ucast: someone already listening on port %d [%s]",
 			  ei->port, ei->interface);
-			LOG(PIL_INFO, "ucast: UDP read process exiting");
+			PILCallLog(LOG, PIL_INFO, "ucast: UDP read process exiting");
 			close(sockfd);
 			cleanexit(0);
 		}
 #else
-		LOG(PIL_CRIT, "ucast: unable to bind socket. Giving up: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: unable to bind socket. Giving up: %s",
 			strerror(errno));
 		close(sockfd);
 		return -1;
 #endif
 	}
 	if (fcntl(sockfd,F_SETFD, FD_CLOEXEC) < 0) {
-		LOG(PIL_CRIT, "ucast: error setting close-on-exec flag: %s",
+		PILCallLog(LOG, PIL_CRIT, "ucast: error setting close-on-exec flag: %s",
 			strerror(errno));
 	}
 	return sockfd;
@@ -649,12 +647,12 @@ static struct ip_private* new_ip_interface(const char *ifn,
  	 * Added by Brian TInsley <btinsley@emageon.com>
  	 */
 	if (!(h = gethostbyname(hbaddr))) {
-		LOG(PIL_CRIT, "ucast: cannot resolve hostname");
+		PILCallLog(LOG, PIL_CRIT, "ucast: cannot resolve hostname");
 		return NULL;
 	}
 
 	if (!(ep = (struct ip_private*) MALLOC(sizeof(struct ip_private)))) {
-		LOG(PIL_CRIT, "ucast: memory allocation error (line %d)",
+		PILCallLog(LOG, PIL_CRIT, "ucast: memory allocation error (line %d)",
 			(__LINE__ - 2) );
 		return NULL;
 	}
@@ -664,13 +662,12 @@ static struct ip_private* new_ip_interface(const char *ifn,
 	*/
 	memcpy(&ep->heartaddr, h->h_addr_list[0], sizeof(ep->heartaddr));
 
-	if (!(ep->interface = (char*)MALLOC(strlen(ifn)+1))) {
-		LOG(PIL_CRIT, "ucast: memory allocation error (line %d)",
+	if (!(ep->interface = STRDUP(ifn))) {
+		PILCallLog(LOG, PIL_CRIT, "ucast: memory allocation error (line %d)",
 			(__LINE__ - 2) );
 		FREE(ep);
 		return NULL;
 	}
-	strcpy(ep->interface, ifn);
 	
 	bzero(&ep->addr, sizeof(ep->addr));	/* zero the struct */
 	ep->addr.sin_family = AF_INET;		/* host byte order */
