@@ -1,4 +1,4 @@
-/* $Id: drac3_hash.h,v 1.1.2.1 2004/04/20 08:50:06 alan Exp $ */
+/* $Id: drac3_hash.h,v 1.1.2.3 2004/08/29 23:22:44 alan Exp $ */
 /*
  * Stonith module for Dell DRACIII (Dell Remote Access Card)
  *
@@ -22,7 +22,8 @@
  */
 
 #include <sys/types.h>
+#include <glib.h>
 
-u_int16_t drac3Crc16(const char *str, const int l);
+guint16 drac3Crc16(const char *str, const int l);
 void drac3AuthHash(const char *chall, const char *pass, char *token, int len);
 
