@@ -1,4 +1,4 @@
-/* $Id: apphbd.c,v 1.51.2.1 2004/04/20 05:05:37 alan Exp $ */
+/* $Id: apphbd.c,v 1.51.2.8 2004/09/17 08:58:07 alan Exp $ */
 /*
  * apphbd:	application heartbeat daemon
  *
@@ -101,12 +101,16 @@ const char *	cmdname = "apphbd";
 #define		DBGDETAIL	2
 static int	usenormalpoll = TRUE;
 static int	watchdogfd = -1;
+static int	cmdline_debug_opt = 0;
 
-#define CONFIG_FILE "./apphbd.cf"
+#ifndef SYSCONFDIR
+#	define	SYSCONFDIR "/etc"
+#endif
+#define CONFIG_FILE SYSCONFDIR "/apphbd.cf"
 #define MAXLINE 128
 #define EOS '\0'
 
-#define DEFAULT_DEBUG_LEVEL	"3"
+#define DEFAULT_DEBUG_LEVEL	"0"
 #define DEFAULT_WDT_DEV		NULL
 #define DEFAULT_WDT_INTERVAL_MS	"1000"
 #define DEFAULT_REALTIME	"yes"
@@ -353,7 +357,7 @@ apphb_client_register(apphb_client_t* client, void* Msg,  size_t length)
 	if (apphbd_config.debug_level >= DBGMIN) {
 		cl_log(LOG_DEBUG
 		,	"apphb_client_register: client: [%s]/[%s] pid %ld"
-		" (uid,gid) = (%ld,%ld)\n"
+		" (uid,gid) = (%ld,%ld)"
 		,	client->appname
 		,	client->appinst
 		,	(long)client->pid
@@ -376,10 +380,8 @@ static void
 apphb_client_remove(gpointer Client)
 {
 	apphb_client_t* client = Client;
-	cl_log(LOG_INFO, "apphb_client_remove: client: %ld\n"
-	,	(long)client->pid);
 	if (apphbd_config.debug_level >= DBGMIN) {
-		cl_log(LOG_DEBUG, "apphb_client_remove: client pid: %ld\n"
+		cl_log(LOG_DEBUG, "apphb_client_remove: client pid: %ld"
 		,	(long)client->pid);
 	}
 	if (client->timerid) {
@@ -895,7 +897,7 @@ main(int argc, char ** argv)
 				cfgfile = optarg;
 				break;
 			case 'd':
-				++apphbd_config.debug_level;
+				++cmdline_debug_opt;
 				break;
 			default:
 				++argerr;
@@ -924,6 +926,14 @@ main(int argc, char ** argv)
 	}
 
 	init_config(cfgfile);
+	/* -d level on command line overrides that in config file */
+	if (cmdline_debug_opt) {
+		apphbd_config.debug_level = cmdline_debug_opt;
+	}
+	if (apphbd_config.debug_level) {
+		cl_log(LOG_DEBUG, "Debug level is %d"
+		,	apphbd_config.debug_level);
+	}
 	return init_start();
 
 }
@@ -1030,7 +1040,6 @@ init_start()
 	drop_privs(0, 0); /* Become nobody */
 	g_main_run(mainloop);
 	return_to_orig_privs();
-	cl_log(LOG_DEBUG, "testing write to syslog when i am a nobody");
 	close_watchdog();
 	wconn->ops->destroy(wconn);
 	if (unlink(PIDFILE) == 0) {
@@ -1049,7 +1058,7 @@ make_daemon(void)
 	pid = fork();
 
 	if (pid < 0) {
-		cl_log(LOG_CRIT, "cannot start daemon.\n");
+		cl_log(LOG_CRIT, "cannot start daemon");
 		exit(LSB_EXIT_GENERIC);
 	}else if (pid > 0) {
 		exit(LSB_EXIT_OK);
@@ -1211,7 +1220,7 @@ close_watchdog(void)
 		if (write(watchdogfd, "V", 1) != 1) {
 			cl_log(LOG_CRIT
 			,	"Watchdog write magic character failure:"
-			" closing watchdog!\n");
+			" closing watchdog!");
 		}
 		close(watchdogfd);
 		watchdogfd=-1;
@@ -1224,7 +1233,7 @@ tickle_watchdog(void)
 	if (watchdogfd >= 0) {
 		if (write(watchdogfd, "", 1) != 1) {
 			cl_log(LOG_CRIT
-			,	"Watchdog write failure: closing watchdog!\n");
+			,	"Watchdog write failure: closing watchdog!");
 			close_watchdog();
 			watchdogfd=-1;
 		}
